#!/bin/sh
#
# Configures to build the sidplayer
#

PLATFORM=linux-generic-g++
SHARING=shared
DEBUG=

# Parse the arguments, setting things to "yes" or "no".

while [ -n "$1" ]; do
   case $1 in
   -platform|-xplatform) # No difference since we don't need to build moc, etc.
        shift; PLATFORM=$1
	;;
   -release)
       DEBUG=
	;;
   -debug)
       DEBUG=-debug
	;;
   -shared)
       SHARING=shared
	;;
   -static)
       SHARING=static
	;;
    *)
	HELP=yes;;
   esac
   shift
done

TARGET=../configs/$PLATFORM-$SHARING$DEBUG

if [ '!' -f $TARGET ]
then
    if [ -f configs/linux-$PLATFORM-g++-$SHARING$DEBUG ]
    then
	TARGET=configs/linux-$PLATFORM-g++-$SHARING$DEBUG
    else
	echo
	echo '    The specified platform/compiler is not supported: ' $TARGET
	echo
	exit 2
    fi
fi

# Next, emit a usage message if something failed.

if [ "$HELP" = "yes" ]; then
    cat <<EOF
Usage: $0 [-debug] [-release] [-shared] [-static] [-platform ...]

The defaults (*) are usually acceptable.  Here is a short explanation of
each option:

 *  -release ........... Compile and link Qt with debugging turned off.
    -debug ............. Compile and link Qt with debugging turned on.

 *  -shared ............ Create and use a shared Qt library (libqt.so)
    -static ............ Create and use a static Qt library (libqt.a)

    -platform target ... The platform you are building on ($PLATFORM)
EOF
   exit 0;
fi


# Hack for libsidplay configuration; assumes $PLATFORM is compiler prefix part
(
##cd libsidplay
##./configure
##cd ..

if [ "$PLATFORM" = linux-generic-g++ ]
then
    echo configuring libsidplay for x86

    sed -e 's/^\(CC=\).*/\1"c++"/' <libsidplay/libtool >libsidplay/libtool_
    mv libsidplay/libtool_ libsidplay/libtool

    sed -e 's/^\(CC =\).*/\1 gcc/' -e 's/^\(CXX =\).*/\1 c++/' <libsidplay/src/Makefile >libsidplay/src/Makefile_
    mv libsidplay/src/Makefile_ libsidplay/src/Makefile

    sed -e 's/#undef SID_FPUFILTER/#define SID_FPUFILTER/' <libsidplay/src/libcfg.h >libsidplay/src/libcfg.h_
    mv libsidplay/src/libcfg.h_ libsidplay/src/libcfg.h

    sed -e 's/#undef SID_FPUMIXERINIT/#define SID_FPUMIXERINIT/' <libsidplay/src/libcfg.h >libsidplay/src/libcfg.h_
    mv libsidplay/src/libcfg.h_ libsidplay/src/libcfg.h
elif [ "$PLATFORM" = linux-ipaq-g++ ]
then
    echo configuring libsidplay for arm

    sed -e 's/^\(CC=\).*/\1"arm-linux-c++"/' <libsidplay/libtool >libsidplay/libtool_
    mv libsidplay/libtool_ libsidplay/libtool

    sed -e 's/^\(CC =\).*/\1 arm-linux-gcc/' -e 's/^\(CXX =\).*/\1 arm-linux-c++/' <libsidplay/src/Makefile >libsidplay/src/Makefile_
    mv libsidplay/src/Makefile_ libsidplay/src/Makefile

    # huge speedup when using SID-filters
    sed -e 's/#define SID_FPUFILTER/#undef SID_FPUFILTER/' <libsidplay/src/libcfg.h >libsidplay/src/libcfg.h_
    mv libsidplay/src/libcfg.h_ libsidplay/src/libcfg.h

    # improves startup time by 2 seconds on the iPAQ
    sed -e 's/#define SID_FPUMIXERINIT/#undef SID_FPUMIXERINIT/' <libsidplay/src/libcfg.h >libsidplay/src/libcfg.h_
    mv libsidplay/src/libcfg.h_ libsidplay/src/libcfg.h
else
    echo
    echo '    The specified platform/compiler is not supported: ' $TARGET
    echo
fi
)

##cd libsidplay
##make
##cd ..


echo Creating makefiles...

a=sidplayer

N=Makefile
M=Makefile.in

if [ -f $TMAKEPATH/tmake.conf ]
then
    # You have tmake. We'll regenerate the file for you...
    H=..
    f=`basename $a`
    (TMAKEPATH=$QTDIR/tmake:$TMAKEPATH tmake CONFIG+=embedded -t $H/qt/tmake/propagate.t -o Makefile.in $f.pro)
fi

cat > $N <<EOF
#############################################################################
# Automatically generated from $M
# Build options from $1
#############################################################################

EOF

cat $TARGET $M >> $N

MAKE=make
echo "sidplayer is now configured for building. Just run $MAKE"
echo "To reconfigure, run $MAKE clean and configure."
