/**************************************************************************************94x78**
**
** This file may be distributed and/or modified under the terms of the GNU General Public
** License version 2 as published by the Free Software Foundation and appearing in the file
** LICENSE.GPL included in the packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE WARRANTY OF DESIGN,
** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
*********************************************************************************************/
#include "sidplayer.h"
#include "builddate.h"
#include "volumebarwidget.h"

#include <qmessagebox.h>
#include <qpushbutton.h>
#include <qtoolbutton.h>
#include <qlcdnumber.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <qslider.h>
#include <qradiobutton.h>
#include <qcheckbox.h>
#include <qheader.h> // must be included before qlistview.h because of bug in qt-2.3.1
#include <qlistview.h>
#include <qtextstream.h>
#include <qdir.h>
#include <qtabwidget.h>
#include <qlayout.h>

#include <qpe/applnk.h>
#include <qpe/resource.h>
#include <qpe/config.h>
#ifdef Q_WS_QWS
#include <qpe/qcopenvelope_qws.h>
#endif


float ItemBase::blend = 0.0;
float ItemBase::dimm  = 0.0;

ItemBase::ItemBase( QListView *parent )
    : QListViewItem( parent )
{
    if ( blend == 0.0 )
    {
        // try to determine the device we are running on
        QFile f( "/proc/cpuinfo" );
        if ( f.open( IO_ReadOnly ) )
        {
            QTextStream ts( &f );
            QString line;
            while( line = ts.readLine() )
                if ( line.contains( "Hardware" ) )
                {
                    if ( line.contains( "Sharp" ) )
                    {
                        blend = 0.866;
                        dimm  = 0.95;
                    }
                    else if ( line.contains( "H3600" ) )
                    {
                        blend = 0.25;
                        dimm  = 1.0;
                    }
                    break;
                }
            if ( blend == 0.0 )
            {
                blend = 0.866;
                dimm  = 0.95;
            }
        }
        f.close();
    }
}

ItemBase::ItemBase( QListView *parent, QListViewItem *after )
    : QListViewItem( parent, after )
{}

//============================================================================================
//============================================================================================

Sidplayer::Sidplayer( int argc, char **argv, QPEApplication *app,
                      QWidget *parent, const char *name, WFlags f )
    : TabbedDialog( parent, name, f ),
      application( app ),
      selectedSong( 1 ),
      totalSongs( 1 ),
      pTune( NULL ),
      pBuffer( NULL ),
      vol1( 256 ),  vol2( 256 ),  vol3( 256 ),  vol4( 256 ),
      pan1L( 255 ), pan2L( 192 ), pan3L( 255 ), pan4L( 192 ),
      pan1R( 192 ), pan2R( 255 ), pan3R( 192 ), pan4R( 255 ),
      allowPopulate( true ),
      audioDeviceIsOpen( false ),
      frgs( 8 ),
      szbs( 13 ),
      playlistChanged( false ),
      playlistSortingColumn( -1 )
{
    // try to determine the device we are running on
    QFile f( "/proc/cpuinfo" );
    if ( f.open( IO_ReadOnly ) )
    {
        QTextStream ts( &f );
        QString line;
        while( line = ts.readLine() )
            if ( line.contains( "Hardware" ) )
            {
                if ( line.contains( "Sharp" ) )
                {
                    frgs = 32;
                    szbs = 11;
                }
                else if ( line.contains( "H3600" ) )
                {
                    frgs = 8;
                    szbs = 13;
                }
                break;
            }
    }
    f.close();

    stateBG = new QButtonGroup();
    stateBG->insert( StopButton );
    stateBG->insert( PauseButton );
    stateBG->insert( PlayButton );
    stateBG->setExclusive( true );

    speedBG = new QButtonGroup();
    speedBG->insert( Speed1x );
    speedBG->insert( Speed2x );
    speedBG->insert( Speed4x );
    speedBG->insert( Speed10x );
    speedBG->setExclusive( true );

    memoryModeBG = new QButtonGroup();
    memoryModeBG->insert( BankSwitching );
    memoryModeBG->insert( TransparentROM );
    memoryModeBG->insert( PlaySidEnv );
    memoryModeBG->setExclusive( true );

    clockSpeedBG = new QButtonGroup();
    clockSpeedBG->insert( PAL );
    clockSpeedBG->insert( NTSC );
    clockSpeedBG->setExclusive( true );

    engineLayout->setStretchFactor( GeneralSettingsBox, 3);
    engineLayout->setStretchFactor( EngineLowerLayout, 2);

    FileView->header()->hide();
    FileView->addColumn( "Files" );
    FileView->setFocus();

    Time->display( " 0:00" );

#ifdef Q_WS_QWS
    //showMaximized(); // is now done by showMainWidget() and would only confuse CanFastload
#else
    //show(); // moved to main.cpp for easier switching between nonmaximized & maximized
#endif

    if ( !myEmuEngine )
    {
        QMessageBox::critical( this, tr("SID Player"),
                               tr("Can not instantiat emuEngine.\nNot enough memory."),
                               tr("&Quit") );
        exit(-1);
    }

    myEmuEngine.getConfig( myEmuConfig );

    VersionLabel->setText( myEmuEngine.getVersionString() );
    BuildLabel->setText( BUILD_DATE );

    // Default audio settings.
    //myEmuConfig.frequency     = 48000;
    //myEmuConfig.bitsPerSample = SIDEMU_16BIT; // SIDEMU_8BIT, SIDEMU_16BIT
    //myEmuConfig.channels      = SIDEMU_STEREO; // SIDEMU_MONO, SIDEMU_STEREO

    //cout << "mos8580: " << myEmuConfig.mos8580 << endl
    //     << "measVol: " << myEmuConfig.measuredVolume << endl
    //     << "emuFilt: " << myEmuConfig.emulateFilter << endl
    //     << "memMode: " << hex << myEmuConfig.memoryMode << dec << endl
    //     << "clkSpee: " << myEmuConfig.clockSpeed << endl
    //     << "forceSS: " << myEmuConfig.forceSongSpeed << endl;

    myEmuConfig.emulateFilter  = true;  // these are
    myEmuConfig.measuredVolume = true;  // are the
    myEmuConfig.mos8580        = false; // defaults
    myEmuConfig.forceSongSpeed = false; // anyway

    myEmuConfig.memoryMode = MPU_BANK_SWITCHING; // defaults
    myEmuConfig.clockSpeed = SIDTUNE_CLOCK_PAL;  // too

    myEmuConfig.volumeControl = SIDEMU_FULLPANNING;
    myEmuConfig.autoPanning   = SIDEMU_NONE;

    // Instantiate the audio driver. The capabilities of the audio driver
    // can override the settings of the SID emulator.
    if ( !myAudio.IsThere() )
    {
        QMessageBox::critical( this, tr("SID Player"),
                               tr("No audio device available!"),
                               tr("&Quit") );
        exit(-1);
    }

    playbackTimer.stop();
    connect( &playbackTimer, SIGNAL( timeout() ), this, SLOT( playbackTimerTimeout() ) );
    volumeTimer.stop();
    connect( &volumeTimer, SIGNAL( timeout() ), this, SLOT( volumeTimerTimeout() ) );
    shutdownTimer.stop();
    connect( &shutdownTimer, SIGNAL( timeout() ), this, SLOT( shutdownTimerTimeout() ) );

    Pan1->setValue( pan1R );
    Pan2->setValue( 511 - pan2L );
    Pan3->setValue( pan3R );
    Pan4->setValue( 511 - pan4L );

    // FileMenu
    FileMenu = new QPopupMenu( this );

    FileMenu->insertItem( loadSmooth("sidplayer/popup/defaults"),
                          "Reload File View", this, SLOT( reloadFileView() ) );
    FileMenu->insertItem( loadSmooth("sidplayer/popup/kill"),
                          "Delete Selected File", this, SLOT( deleteFile() ) );
    FileMenu->insertSeparator(); // ----------------------------------------------------------
    FileMenu->insertSeparator(); // ----------------------------------------------------------
    showParentDirEntryID = FileMenu->insertItem( "Show Parent Dir Entry", this,
                                                 SLOT( showParentDirEntry() ) );
    showHiddenEntriesID = FileMenu->insertItem( "Show Hidden Entries", this,
                                                 SLOT( showHiddenEntries() ) );
    FileMenu->insertSeparator(); // ----------------------------------------------------------
    homeButtonTogglesViewID = FileMenu->insertItem( "Home Button Toggles View", this,
                                                    SLOT( homeButtonTogglesView() ) );
    disableHWButtonsID = FileMenu->insertItem( "Disable Hardware Buttons", this,
                                               SLOT( disableHWButtons() ) );

    connect( FileMenu, SIGNAL( aboutToHide() ), this, SLOT( onFileMenuHide() ) );

    // PlaylistMenu
    PlaylistMenu = new QPopupMenu( this );

    PlaylistMenu->insertItem( loadSmooth("sidplayer/popup/tune_insert"),
                              "Insert Current Tune", this, SLOT( insertTune() ) );
    PlaylistMenu->insertItem( loadSmooth("sidplayer/popup/tune_remove"),
                              "Remove Selected Tune", this, SLOT( removeTune() ) );
    PlaylistMenu->insertSeparator(); // ------------------------------------------------------
    PlaylistMenu->insertItem( loadSmooth("sidplayer/popup/dir_insert"),
                              "Insert Current Dir", this, SLOT( insertDir() ) );
    PlaylistMenu->insertItem( loadSmooth("sidplayer/popup/dir_recursive"),
                              "Insert Dir Recursive", this, SLOT( insertDirRecursive() ) );
    PlaylistMenu->insertSeparator(); // ------------------------------------------------------
    PlaylistMenu->insertItem( loadSmooth("sidplayer/popup/kill"),
                              "Clear Playlist", this, SLOT( clearList() ) );
    PlaylistMenu->insertItem( loadSmooth("sidplayer/popup/shuffle"),
                              "Shuffle Playlist", this, SLOT( shuffleList() ) );
    PlaylistMenu->insertSeparator(); // ------------------------------------------------------
    PlaylistMenu->insertSeparator(); // ------------------------------------------------------
    allowSortingID = PlaylistMenu->insertItem( "Allow Column Sorting", this,
                                               SLOT( allowSorting() ) );
    PlaylistView->setSorting(-1);
    allowMovingID = PlaylistMenu->insertItem( "Allow Up/Down Moving", this,
                                              SLOT( allowMoving() ) );
    PlaylistMenu->insertSeparator(); // ------------------------------------------------------
    consistentFileViewID = PlaylistMenu->insertItem( "Consistent File View", this,
                                                     SLOT( consistentFileView() ) );

    connect( PlaylistMenu, SIGNAL( aboutToHide() ), this, SLOT( onListMenuHide() ) );

    FileView->installEventFilter( this );
    PlaylistView->installEventFilter( this );
    this->installEventFilter( this );

    pmef = new PopupMenuEventFilter();
    FileMenu->installEventFilter( pmef );
    PlaylistMenu->installEventFilter( pmef );

    tdef = new TimeDisplayEventFilter();
    Time->installEventFilter( tdef );
    SpeedLabel->installEventFilter( tdef );
    connect( tdef, SIGNAL( mousePressed() ), this, SLOT( timeDisplayMousePressed() ) );
    connect( tdef, SIGNAL( mouseReleased() ), this, SLOT( timeDisplayMouseReleased() ) );

#ifdef WITH_TAB_ICONS
    connect( (QObject*) TabWidget, SIGNAL( currentChanged(QWidget*) ),
             this, SLOT( tabWidgetCurrentChanged(QWidget*) ) );

    TabWidget->changeTab( file,   loadSmooth( "sidplayer/tabbar/file"   ), "" );
    TabWidget->changeTab( list,   loadSmooth( "sidplayer/tabbar/list"   ), "" );
    TabWidget->changeTab( mixer,  loadSmooth( "sidplayer/tabbar/mixer"  ), "" );
    TabWidget->changeTab( engine, loadSmooth( "sidplayer/tabbar/engine" ), "" );
    TabWidget->changeTab( info,   loadSmooth( "sidplayer/tabbar/info"   ), "" );
#endif

#ifdef WITH_COLOR_ICONS
    // Whoah!! This needs to be specified to get rid of the string inserted by Designer --
    // even if the text is just an empty string ("").
    StopButton->setText( QString::null );
    StopButton->setIconSet( loadIconSet( "sidplayer/buttons/stop" ) );
    PauseButton->setText( QString::null );
    PauseButton->setIconSet( loadIconSet( "sidplayer/buttons/pause" ) );
    PlayButton->setText( QString::null );
    PlayButton->setIconSet( loadIconSet( "sidplayer/buttons/play" ) );

    ParentDirButton->setText( QString::null );
    ParentDirButton->setFixedSize( 20, 16 );
    //ParentDirButton->setPixmap( Resource::loadPixmap( "sidplayer/buttons/dummy1x1" ) );

    PrevButton->setText( QString::null );
    PrevButton->setIconSet( loadIconSet( "sidplayer/buttons/prev" ) );
    NextButton->setText( QString::null );
    NextButton->setIconSet( loadIconSet( "sidplayer/buttons/next" ) );
    UpButton->setText( QString::null );
    UpButton->setIconSet( loadIconSet( "sidplayer/buttons/up" ) );
    DownButton->setText( QString::null );
    DownButton->setIconSet( loadIconSet( "sidplayer/buttons/down" ) );

    FileMenuButton->setPixmap( loadSmooth( "sidplayer/buttons/menu" ) );
    ListMenuButton->setPixmap( loadSmooth( "sidplayer/buttons/menu" ) );

    EngineDefaultsButton->setIconSet( loadIconSet( "sidplayer/popup/defaults" ) );
#endif

    connect( PlaylistView->header(), SIGNAL( clicked(int) ), // to correctly set the oddFlag
             this, SLOT( headerClicked(int) ) );             // after column sorting

    QPixmap SplashPix = Resource::loadPixmap( "sidplayer/splashlogo" );
    SplashLabel = new QLabel( file );
    SplashLabel->setAlignment( AlignCenter );
    SplashLabel->setPixmap( SplashPix );
    // the rest is done in Sidplayer::showEvent()

    readConfig();
    readPlaylistSectionOrder();
    readPopupMenusSettings();

    PlaylistView->blockSignals( true );
    loadPlaylist();
    PlaylistView->blockSignals( false );
    checkButtonState();

    PlaylistView->setColumnAlignment( 3, AlignHCenter ); // center Songs column

    Config cfg("Sidplayer");
    cfg.setGroup("List");
    if ( PlaylistMenu->isItemChecked( allowSortingID ) )
    {
        playlistSortingColumn = cfg.readNumEntry( "sortingColumn", -1 );
        PlaylistView->setSorting( playlistSortingColumn );
        setAllPlaylistViewItemsOddFlag();
    }

    cfg.setGroup("Path");

    currentDir = cfg.readEntry( "currentDir", "" );
    QString selectedEntry = cfg.readEntry( "selectedEntry", "" );

    cfg.setGroup("Various");

    listeningMileage = QString( cfg.readEntry( "listeningMileage", "0" ) ).toLong( NULL );
    updateListeningMileageLabel( true );

#if 1
    if ( QFileInfo( selectedEntry ).isFile() )
    {
        populateFileView( QDir::cleanDirPath( currentDir ), selectedEntry );
        FileView->scrollBy( 0, -1 ); // strange - I have no idea why this is necessary
        loadTune( selectedEntry ); // sets selectedSong
        setCaptionAccordingToSong(); // uses selectedSong
        PlayButton->setEnabled( true );
    }
    else if ( QDir( currentDir ).exists() )
    {
        populateFileView( QDir::cleanDirPath( currentDir ), selectedEntry );
        FileView->scrollBy( 0, -1 ); // strange - I have no idea why this is necessary
    }
    else
        populateFileView( QDir::cleanDirPath( QPEApplication::documentDir() ) );
    argc = argc; // use the parameter to satisfy the compiler
    argv = argv; // use the parameter to satisfy the compiler
#else
    // when clicking on an icon the program is started with  -qcop /tmp/qcop-msg-sidplayer
    // so the following does not really work
    if ( argc < 2 )
        populateFileView( QDir::currentDirPath() );
    else
    {
        QFileInfo fileinfo = QFileInfo( argv[1] );
        if ( fileinfo.isDir() )
            populateFileView( fileinfo.dirPath() );
        else
            setDocument( argv[1] );
    }
#endif

    if ( !audioDeviceIsOpen )
        if ( !openAudio() )
        {
            QMessageBox::critical( this, tr("SID Player"),
                                   tr("audioDriver:\n") + myAudio.GetErrorString(),
                                   tr("&Quit") );
            exit(-1);
        }
    // Release it at startup in any case, otherwise the device will be aquired when checking
    // "fast load" and won't be released until SIDPlayer ist opened and closed the first time.
    //if ( ReleaseAudioDevice->isChecked() )
        closeAudio();
    //cout << "conf begin" << endl;
    myEmuEngine.setConfig( myEmuConfig );
    //cout << "conf end" << endl;

    myEmuEngine.setVoiceVolume( 1, pan1L, pan1R, vol1 );
    myEmuEngine.setVoiceVolume( 2, pan2L, pan2R, vol2 );
    myEmuEngine.setVoiceVolume( 3, pan3L, pan3R, vol3 );
    myEmuEngine.setVoiceVolume( 4, pan4L, pan4R, vol4 );

    FPSLabel_2->setText( "Latency:" );
    FPSLabel->setText( QString::number( myAudio.GetBlockSize() * 8 * // bit
                                        myAudio.GetFragments() * 1000 / // ms
                                        myEmuConfig.frequency /
                                        myEmuConfig.channels /
                                        myEmuConfig.bitsPerSample ) + " ms" );
}

Sidplayer::~Sidplayer()
{
    stop();

    myAudio.Reset();
    if ( pBuffer )
        delete[] pBuffer;
}

//--------------------------------------------------------------------------------------------

QPixmap Sidplayer::loadSmooth( const QString &name )
{
    QImage image = Resource::loadImage( name );
    QPixmap pixmap;
    pixmap.convertFromImage( image.smoothScale( 14, 14 ) );
    return pixmap;
}

QIconSet Sidplayer::loadIconSet( const QString &name )
{
    QImage image = Resource::loadImage( name );
    QPixmap pixmap;
    pixmap.convertFromImage( image );
    QIconSet iconSet( pixmap );

#ifndef QT_NO_DEPTH_32	// have alpha-blended pixmaps
    QImage disabledImage( image.width(), image.height(), 32 );
    disabledImage.setAlphaBuffer( true );
    for ( int y = 0; y < disabledImage.height(); ++y )
        for ( int x = 0; x < disabledImage.width(); ++x )
        {
            QRgb pixel = image.pixel( x, y );
            int r, g, b;
            //r = qRed( pixel );
            //g = qGreen( pixel );
            //b = qBlue( pixel );
            r = g = b = ( qRed( pixel ) + qGreen( pixel ) + qBlue( pixel ) ) / 3;
            int a = qAlpha( pixel ) / 2;
            disabledImage.setPixel( x, y, qRgba( r, g, b, a ) );
        }
    QPixmap disabledPixmap;
    disabledPixmap.convertFromImage( disabledImage );
    if ( !disabledPixmap.isNull() )
        iconSet.setPixmap( disabledPixmap, QIconSet::Small, QIconSet::Disabled );
#endif

    return iconSet;
}

//--------------------------------------------------------------------------------------------

void Sidplayer::loadTune( const char* fileName )
{
    if ( SplashLabel )
    {
        delete SplashLabel;
        SplashLabel = NULL;
    }

    if ( pTune )
        delete pTune;
    pTune = new sidTune( fileName );
    pTune->getInfo( myTuneInfo );
    if ( !*pTune )
    {
        QMessageBox::critical( this, tr("SID Player"),
                               tr("sidTuneInfo:\n") + myTuneInfo.statusString,
                               tr("&Quit") );
        exit(-1);
    }
    selectedSong = myTuneInfo.startSong;
    totalSongs = myTuneInfo.songs;
    SongNumber->blockSignals( true ); // I know, not very elegant ...
    SongNumber->setMaxValue( totalSongs );
    SongNumber->setValue( selectedSong );
    SongNumber->blockSignals( false ); // I know, not very elegant ...

    if ( myTuneInfo.numberOfInfoStrings == 3 )
    {
        NameLabel->setText( myTuneInfo.nameString );
        AuthorLabel->setText( myTuneInfo.authorString );
        CopyrightLabel->setText( myTuneInfo.copyrightString );
    }
    //else
    //{
    //    for ( int infoi = 0; infoi < myTuneInfo.numberOfInfoStrings; infoi++ )
    //        cout << "Description  : " << myTuneInfo.infoString[infoi] << endl;
    //}
    if ( myTuneInfo.songs == 1 )
    {
        SongsLabel->setText( "1" );
        SongNumber->setEnabled( false );
    }
    else
    {
        SongsLabel->setText( QString::number( myTuneInfo.songs ) + " (Startsong: " +
                             QString::number( myTuneInfo.startSong ) + ")" );
        SongNumber->setEnabled( true );
    }
    AddressesLabel->setText(
        "$" + QString::number( myTuneInfo.loadAddr, 16 ).rightJustify( 4, '0' ) + ", " +
        "$" + QString::number( myTuneInfo.initAddr, 16 ).rightJustify( 4, '0' ) + ", " +
        "$" + QString::number( myTuneInfo.playAddr, 16 ).rightJustify( 4, '0' ) );
    FormatLabel->setText( myTuneInfo.formatString );
    FilesizeLabel->setText( QString::number( myTuneInfo.dataFileLen ) + " Byte" );//c64dataLen
}

void Sidplayer::initializeEngine( uword songNumber )
{
    stop();

    myEmuEngine.setVoiceVolume( 1, pan1L, pan1R, vol1 );
    myEmuEngine.setVoiceVolume( 2, pan2L, pan2R, vol2 );
    myEmuEngine.setVoiceVolume( 3, pan3L, pan3R, vol3 );
    myEmuEngine.setVoiceVolume( 4, pan4L, pan4R, vol4 );

    if ( !sidEmuInitializeSong( myEmuEngine, *pTune, songNumber ) )
    {
        QMessageBox::critical( this, tr("SID Player"),
                               tr("SID Emulator Engine\ncomponents not ready."),
                               tr("&Quit") );
        exit(-1);
    }
    selectedSong = songNumber;

    // Read out the current settings of the sidtune.
    pTune->getInfo( myTuneInfo );
    if ( !*pTune )
    {
        QMessageBox::critical( this, tr("SID Player"),
                               tr("sidTuneInfo:\n") + myTuneInfo.statusString,
                               tr("&Quit") );
        exit(-1);
    }

    SongSpeedLabel->setText( myTuneInfo.speedString );
    //cout << "Setting song : " << myTuneInfo.currentSong << endl;
}

void Sidplayer::setCaptionAccordingToSong()
{
    if ( myTuneInfo.songs == 1 )
        setCaption( myTuneInfo.nameString + QString(" - ") + myTuneInfo.authorString );
    else
        setCaption( "[" + QString::number( selectedSong ) + "/" +
                    QString::number( myTuneInfo.songs ) + "] " +
                    myTuneInfo.nameString + " - " + myTuneInfo.authorString );
}

void Sidplayer::setDocument( const QString& fileref ) // dedicated (magic) function name
{
    QString file;
    DocLnk link( fileref );
    if ( link.isValid() )
        file = link.file();
    else
        file = fileref;

    QFileInfo fileinfo( file );
    if ( fileinfo.isReadable() )
        loadTune( file );
    else
    {
        if ( allowPopulate )
            populateFileView( fileinfo.dirPath() );
        return;
    }

    if ( allowPopulate )
    {
        if ( fileinfo.dirPath() != currentDir )
            populateFileView( fileinfo.dirPath() );
        selectFileItem( file );
    }
    allowPopulate = true;

    // call the slot manually
    songNumberChanged( selectedSong );
}

//--------------------------------------------------------------------------------------------

bool Sidplayer::openAudio()
{
    Config cfg("Sidplayer");
    cfg.setGroup("Audio");
    udword frequency     = cfg.readNumEntry( "frequency",   48000 ); // 48000
    int    bitsPerSample = cfg.readNumEntry( "bitsPerSample",  16 ); // 8 / 16
    int    channels      = cfg.readNumEntry( "channels",        2 ); // 1 / 2
    int    fragments     = cfg.readNumEntry( "fragments",    frgs ); //  8 | 32 | play with
    int    fragSizeBase  = cfg.readNumEntry( "fragSizeBase", szbs ); // 13 | 11 | these values

    if ( !myAudio.Open( frequency, bitsPerSample, channels, fragments, fragSizeBase ) )
    {
        QMessageBox::critical( this, tr("SID Player"),
                               tr("audioDriver:\n") + myAudio.GetErrorString(),
                               tr("&Close") );
        return false;
        //exit(-1);
    }

    // only write when necessary
    if ( !cfg.hasKey( "frequency" ) || frequency != myAudio.GetFrequency() )
    {
        frequency = myAudio.GetFrequency();
        cfg.writeEntry( "frequency", (int) frequency ); // cast to int possibly dangerous
    }
    if ( !cfg.hasKey( "bitsPerSample" ) || bitsPerSample != myAudio.GetSamplePrecision() )
    {
        bitsPerSample = myAudio.GetSamplePrecision();
        cfg.writeEntry( "bitsPerSample", (int) bitsPerSample );
    }
    if ( !cfg.hasKey( "channels" ) || channels != myAudio.GetChannels() )
    {
        channels = myAudio.GetChannels();
        cfg.writeEntry( "channels", (int) channels );
    }

    // different here
    if ( !cfg.hasKey( "fragments" ) )
        cfg.writeEntry( "fragments", (int) fragments );
    if ( !cfg.hasKey( "fragSizeBase" ) )
        cfg.writeEntry( "fragSizeBase", (int) fragSizeBase );
    fragments = myAudio.GetFragments();
    fragSizeBase = myAudio.GetFragSizeBase();

    audioDeviceIsOpen = true;

    if ( !pBuffer )
    {
        //cout << "--------------------------------------------------" << endl;
        //cout << "FragSizeBase   : " << myAudio.GetFragSizeBase() << endl;

        FrequencyLabel->setText( QString::number( frequency / 1000. ) + " kHz" );
        switch ( channels )
        {
        case 1:
            ChannelsLabel->setText( "1 (mono)" );
            break;
        case 2:
            ChannelsLabel->setText( "2 (stereo)" );
            break;
        }
        PrecisionLabel->setText( QString::number( bitsPerSample ) + " bit" );
        BlockSizeLabel->setText( QString::number( myAudio.GetBlockSize() / 1024. ) + " kB" );
        if ( fragments == -1 ) // e.g. Zaurus
            FragmentsLabel->setText( "no info" );
        else
            FragmentsLabel->setText( QString::number( fragments ) );

        bufSize = myAudio.GetBlockSize();

        if ( ( pBuffer = new ubyte[bufSize] ) == 0 )
        {
            QMessageBox::critical( this, tr("SID Player"),
                                   tr("Can not instantiat Buffer.\nNot enough memory."),
                                   tr("&Quit") );
            exit(-1);
        }

        // ======================================================================
        // CONFIGURE THE EMULATOR ENGINE
        // ======================================================================

        // Configure the SID emulator according to the audio driver settings.
        myEmuConfig.frequency = frequency;
        myEmuConfig.bitsPerSample = bitsPerSample;
        myEmuConfig.channels = channels;
        myEmuConfig.sampleFormat = myAudio.GetSampleEncoding();
    }

    return true;
}

void Sidplayer::closeAudio()
{
    myAudio.Close();
    audioDeviceIsOpen = false;
}

//--------------------------------------------------------------------------------------------

void Sidplayer::updateListeningMileageLabel( bool forceUpdate = false )
{
    static int oldSecondsTotal = 0;
    int secondsTotal = myEmuEngine.getSecondsTotal();
    if ( secondsTotal != oldSecondsTotal || forceUpdate )
    {
        long mileage = listeningMileage + secondsTotal;
        int days = mileage / ( 3600 * 24 );
        mileage -= days * 3600 * 24;
        int hours = mileage / 3600;
        mileage -= hours * 3600;
        int minutes = mileage / 60;
        mileage -= minutes * 60;
        int seconds = mileage;
        ListeningMileageLabel->setText( QString::number( days ) + " d, " +
                                        QString::number( hours ) + " h, " +
                                        QString::number( minutes ) + " m, " +
                                        QString::number( seconds ) + " s" );
        oldSecondsTotal = secondsTotal;
    }
}

void Sidplayer::playbackTimerTimeout()
{
    sidEmuFillBuffer( myEmuEngine, *pTune, pBuffer, bufSize );
    myAudio.Play( pBuffer, bufSize );
    int time = myEmuEngine.getSecondsThisSong();
    Time->display( QString::number( time / 60 ).rightJustify( 2, ' ' ) + ":" +
                   QString::number( time % 60 ).rightJustify( 2, '0' ) );
    updateListeningMileageLabel();

    static const int meanFPSLabelUpdatesPerSecond = 5;
    static int samples = 1;
    static QTime lasttime = QTime();
    static long milliseconds = 0;
    static int i = 0;
    milliseconds += lasttime.restart();
    i++;
    if ( i >= samples && milliseconds )
    {
        int fps = 1000 * samples / milliseconds;
        FPSLabel->setText( QString::number( fps ) + " / s" );
        milliseconds = 0;
        i = 0;
        if ( fps / meanFPSLabelUpdatesPerSecond > samples )
            samples++;
        else
            if ( samples > 1 ) samples--;
    }
}

void Sidplayer::volumeTimerTimeout()
{
/*
    // Qtopia & Opie in newer verisons
    Config cfg( "qpe" );
    cfg.setGroup ( "Volume" );
    QString volumeEntryKey = "VolumePercent";
    if ( ! cfg.hasKey( volumeEntryKey ) )
    {
        // QPE & Qtopia in older versions
        cfg = Config( "Sound" );
        cfg.setGroup( "System" );
        volumeEntryKey = "Volume";
    }

    int volumePercent = cfg.readNumEntry( volumeEntryKey );
    if ( increaseVolume )
    {
        volumePercent += VOLUME_STEP;
        if ( volumePercent > 100 )
            volumePercent = 100;
    }
    else
    {
        volumePercent -= VOLUME_STEP;
        if ( volumePercent < 0 )
            volumePercent = 0;
    }
    cfg.writeEntry( volumeEntryKey, volumePercent );

#if ( defined Q_WS_QWS || defined(_WS_QWS_) ) && !defined(QT_NO_COP)
    // Send notification that the volume has changed
    QCopEnvelope( "QPE/System", "volumeChange(bool)" ) << false; // false means "not muted"
#endif
*/

    // direct method
    int volumePercent = VolumeBar->getVolume();
    if ( increaseVolume )
    {
        volumePercent += VOLUME_STEP;
        if ( volumePercent > 100 )
            volumePercent = 100;
    }
    else
    {
        volumePercent -= VOLUME_STEP;
        if ( volumePercent < 0 )
            volumePercent = 0;
    }
    VolumeBar->setVolume( volumePercent );
}

//--------------------------------------------------------------------------------------------

void Sidplayer::start()
{
#if defined(Q_WS_QWS) && !defined(QT_NO_COP)
    QCopEnvelope( "QPE/System", "setScreenSaverMode(int)" ) << QPEApplication::DisableSuspend;
#endif
    playbackTimer.start(0, false);
    FPSLabel_2->setText( "Update Rate:" );
}

void Sidplayer::stop()
{
#if defined(Q_WS_QWS) && !defined(QT_NO_COP)
    QCopEnvelope( "QPE/System", "setScreenSaverMode(int)" ) << QPEApplication::Enable;
#endif
    playbackTimer.stop();
    myAudio.Reset(); // is OK on the iPAQ
    FPSLabel_2->setText( "Latency:" );
    FPSLabel->setText( QString::number( myAudio.GetBlockSize() * 8 * // bit
                                        myAudio.GetFragments() * 1000 / // ms
                                        myEmuConfig.frequency /
                                        myEmuConfig.channels /
                                        myEmuConfig.bitsPerSample ) + " ms" );
}

//--------------------------------------------------------------------------------------------

#ifdef WITH_TAB_ICONS
void Sidplayer::tabWidgetCurrentChanged( QWidget *child )
{
    if ( child->name() == QString( "file" ) )
    {
        if ( TabWidget->tabLabel( list ) != "" )   TabWidget->changeTab( list, QString( "" ) );
        if ( TabWidget->tabLabel( mixer ) != "" )  TabWidget->changeTab( mixer, QString( "" ) );
        if ( TabWidget->tabLabel( engine ) != "" ) TabWidget->changeTab( engine, QString( "" ) );
        if ( TabWidget->tabLabel( info ) != "" )   TabWidget->changeTab( info, QString( "" ) );
        TabWidget->changeTab( file, QString( "File" ) );
        return;
    }

    if ( child->name() == QString( "list" ) )
    {
        if ( TabWidget->tabLabel( file ) != "" )   TabWidget->changeTab( file, QString( "" ) );
        if ( TabWidget->tabLabel( mixer ) != "" )  TabWidget->changeTab( mixer, QString( "" ) );
        if ( TabWidget->tabLabel( engine ) != "" ) TabWidget->changeTab( engine, QString( "" ) );
        if ( TabWidget->tabLabel( info ) != "" )   TabWidget->changeTab( info, QString( "" ) );
        TabWidget->changeTab( list, QString( "List" ) );
        return;
    }

    if ( child->name() == QString( "mixer" ) )
    {
        if ( TabWidget->tabLabel( file ) != "" )   TabWidget->changeTab( file, QString( "" ) );
        if ( TabWidget->tabLabel( list ) != "" )   TabWidget->changeTab( list, QString( "" ) );
        if ( TabWidget->tabLabel( engine ) != "" ) TabWidget->changeTab( engine, QString( "" ) );
        if ( TabWidget->tabLabel( info ) != "" )   TabWidget->changeTab( info, QString( "" ) );
        TabWidget->changeTab( mixer, QString( "Mixer" ) );
        return;
    }

    if ( child->name() == QString( "engine" ) )
    {
        if ( TabWidget->tabLabel( file ) != "" )   TabWidget->changeTab( file, QString( "" ) );
        if ( TabWidget->tabLabel( list ) != "" )   TabWidget->changeTab( list, QString( "" ) );
        if ( TabWidget->tabLabel( mixer ) != "" )  TabWidget->changeTab( mixer, QString( "" ) );
        if ( TabWidget->tabLabel( info ) != "" )   TabWidget->changeTab( info, QString( "" ) );
        TabWidget->changeTab( engine, QString( "Engine" ) );
        return;
    }

    if ( child->name() == QString( "info" ) )
    {
        if ( TabWidget->tabLabel( file ) != "" )   TabWidget->changeTab( file, QString( "" ) );
        if ( TabWidget->tabLabel( list ) != "" )   TabWidget->changeTab( list, QString( "" ) );
        if ( TabWidget->tabLabel( mixer ) != "" )  TabWidget->changeTab( mixer, QString( "" ) );
        if ( TabWidget->tabLabel( engine ) != "" ) TabWidget->changeTab( engine, QString( "" ) );
        TabWidget->changeTab( info, QString( "Info" ) );
        return;
    }
}
#endif

//--------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------

void Sidplayer::readPopupMenusSettings()
{
    Config cfg("Sidplayer");
    cfg.setGroup("Menus");

    // FileMenu
    FileMenu->setItemChecked( showParentDirEntryID,
                              cfg.readBoolEntry( "showParentDirEntry",    false ) );
    FileMenu->setItemChecked( showHiddenEntriesID,
                              cfg.readBoolEntry( "showHiddenEntries",     true  ) );
    FileMenu->setItemChecked( homeButtonTogglesViewID,
                              cfg.readBoolEntry( "homeButtonTogglesView", true  ) );
    FileMenu->setItemChecked( disableHWButtonsID,
                              cfg.readBoolEntry( "disableHWButtons",      false ) );

    // PlaylistMenu
    PlaylistMenu->setItemChecked( allowSortingID,
                                  cfg.readBoolEntry( "allowSorting",       false ) );
    PlaylistMenu->setItemChecked( allowMovingID,
                                  cfg.readBoolEntry( "allowMoving",        false ) );
    PlaylistMenu->setItemChecked( consistentFileViewID,
                                  cfg.readBoolEntry( "consistentFileView", true  ) );
}

void Sidplayer::writePopupMenusSettings()
{
    Config cfg("Sidplayer");
    cfg.setGroup("Menus");

    bool flag;

    // FileMenu
    flag = FileMenu->isItemChecked( showParentDirEntryID );
    if ( !cfg.hasKey( "showParentDirEntry" ) ||
         cfg.readBoolEntry( "showParentDirEntry" ) != flag )
        cfg.writeEntry( "showParentDirEntry", flag );
    flag = FileMenu->isItemChecked( showHiddenEntriesID );
    if ( !cfg.hasKey( "showHiddenEntries" ) ||
         cfg.readBoolEntry( "showHiddenEntries" ) != flag )
        cfg.writeEntry( "showHiddenEntries", flag );
    flag = FileMenu->isItemChecked( homeButtonTogglesViewID );
    if ( !cfg.hasKey( "homeButtonTogglesView" ) ||
         cfg.readBoolEntry( "homeButtonTogglesView" ) != flag )
        cfg.writeEntry( "homeButtonTogglesView", flag );
    flag = FileMenu->isItemChecked( disableHWButtonsID );
    if ( !cfg.hasKey( "disableHWButtons" ) ||
         cfg.readBoolEntry( "disableHWButtons" ) != flag )
        cfg.writeEntry( "disableHWButtons", flag );

    // PlaylistMenu
    flag = PlaylistMenu->isItemChecked( allowSortingID );
    if ( !cfg.hasKey( "allowSorting" ) ||
         cfg.readBoolEntry( "allowSorting" ) != flag )
        cfg.writeEntry( "allowSorting", flag );
    flag = PlaylistMenu->isItemChecked( allowMovingID );
    if ( !cfg.hasKey( "allowMoving" ) ||
         cfg.readBoolEntry( "allowMoving" ) != flag )
        cfg.writeEntry( "allowMoving", flag );
    flag = PlaylistMenu->isItemChecked( consistentFileViewID );
    if ( !cfg.hasKey( "consistentFileView" ) ||
         cfg.readBoolEntry( "consistentFileView" ) != flag )
        cfg.writeEntry( "consistentFileView", flag );
}
