/**************************************************************************************94x78**
**
** This file may be distributed and/or modified under the terms of the GNU General Public
** License version 2 as published by the Free Software Foundation and appearing in the file
** LICENSE.GPL included in the packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE WARRANTY OF DESIGN,
** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
*********************************************************************************************/
#ifndef SIDPLAYER_H
#define SIDPLAYER_H

#include "tabbeddialog.h"

#include "libsidplay/src/player.h"
#include "libsidplay/src/fformat.h"
#include "libsidplay/src/myendian.h"
#include "audiodrv.h"

#include <qpe/qpeapplication.h>
#include <qtimer.h>
#include <qbuttongroup.h>
#include <qlistview.h>
#include <qfileinfo.h>
#include <qpopupmenu.h>
#include <qradiobutton.h>
#include <qtextstream.h>

#define SOLO_GET_RIGHT_HACK
#define WITH_TAB_ICONS
#define WITH_COLOR_ICONS

#define VOLUME_TIMER_INTERVAL 100 // 200
#define VOLUME_STEP 2             //   4

#define SHUTDOWN_TIMER_TIMEOUT 10 // 2 would probably be also long enough


extern bool sidEmuFastForwardReplay( int );


class PlaylistItem;
class PopupMenuEventFilter;
class TimeDisplayEventFilter;


class Sidplayer : public TabbedDialog
{
    Q_OBJECT

public:
    Sidplayer( int argc, char **argv, QPEApplication *app,
               QWidget *parent = 0, const char *name = 0, WFlags f = 0 );
    ~Sidplayer();

public slots:
    void setDocument( const QString& fileref ); // dedicated (magic) function name

private slots:
    void playbackTimerTimeout();
    void volumeTimerTimeout();
    void shutdownTimerTimeout();
    void playButtonToggled( bool on );
    void stopButtonToggled( bool on );
    void pauseButtonToggled( bool on );
    void songNumberChanged( int number );

    void speed1xToggled( bool on );
    void speed2xToggled( bool on );
    void speed4xToggled( bool on );
    void speed10xToggled( bool on );

    void vol1Changed( int vol );
    void vol2Changed( int vol );
    void vol3Changed( int vol );
    void vol4Changed( int vol );

    void pan1Changed( int pan );
    void pan2Changed( int pan );
    void pan3Changed( int pan );
    void pan4Changed( int pan );

    void mute1Toggled( bool on );
    void mute2Toggled( bool on );
    void mute3Toggled( bool on );
    void mute4Toggled( bool on );

    void solo1Toggled( bool on );
    void solo2Toggled( bool on );
    void solo3Toggled( bool on );
    void solo4Toggled( bool on );

    void sidFilterToggled( bool on );
    void measuredVolumeToggled( bool on );
    void mos8580Toggled( bool on );
    void forceSongSpeedToggled( bool on );
    void stereoSurroundToggled( bool on );
    void autoPanningToggled( bool on );

    void bankSwitchingToggled( bool on );
    void transparentROMToggled( bool on );
    void playSidEnvToggled( bool on );

    void palToggled( bool on );
    void ntscToggled( bool on );

    void engineDefaultsClicked();
    void releaseAudioDeviceToggled( bool on );

    void fileViewPressed( QListViewItem *item );

    void fileMenuButtonPressed();
    void onFileMenuHide();
    void reloadFileView();
    void deleteFile();
    void showParentDirEntry();
    void showHiddenEntries();
    void homeButtonTogglesView();
    void disableHWButtons();

    void parentDirButtonPressed();

    void prevButtonClicked();
    void nextButtonClicked();
    void upButtonClicked();
    void downButtonClicked();
    void playlistViewPressed( QListViewItem *item );

    void listMenuButtonPressed();
    void onListMenuHide();
    void headerClicked( int section );
    void insertTune();
    void removeTune();
    void insertDir();
    void insertDirRecursive();
    void clearList();
    void shuffleList();
    void allowSorting();
    void allowMoving();
    void consistentFileView();

    void timeDisplayMousePressed() { Speed10x->setChecked( true ); }
    void timeDisplayMouseReleased() { Speed1x->setChecked( true ); }

#ifdef WITH_TAB_ICONS
    void tabWidgetCurrentChanged( QWidget *child );
#endif

private:
    QPixmap loadSmooth( const QString &name );
    QIconSet loadIconSet( const QString &name );

    template<class T>
    void Sidplayer::setFollowingItemsOddFlag( T *item )
        {
            if ( item == NULL )
                return;

            evenFlag = item->isOdd;

            QListViewItemIterator iter( item );
            ++iter;
            for ( ; iter.current(); ++iter )
                ( (T *) iter.current() )->isOdd = evenFlag = !evenFlag;
        }

    void setAllFileViewItemsOddFlag();
    void setAllPlaylistViewItemsOddFlag();

    void loadTune( const char *fileName );
    void initializeEngine( uword songNumber );
    void setCaptionAccordingToSong();
    void start();
    void stop();

    bool openAudio();
    void closeAudio();

    void goToParentDir();
    void selectFileItem( const QString &selectThis );
    void fitFileViewColumnWidths();
    void populateFileView( const QString &dir, const QString &selectThis = "" );

    bool eventFilter( QObject *o, QEvent *e ); // dedicated (magic) function name
    void showEvent( QShowEvent * ); // dedicated (magic) function name
    void resizeEvent( QResizeEvent * ); // dedicated (magic) function name
    void closeEvent( QCloseEvent *e ); // dedicated (magic) function name

#ifdef SOLO_GET_RIGHT_HACK
    void soloGetRight( int muteReleased );
#endif

    void updateListeningMileageLabel( bool force = false );

    void checkButtonState();

    void insertDir_( QString dirpath = "" );
    void insertDirRecursive_( QString dirpath = "" );

    void selectPlaylistItem( const QString &selectThis );
    void loadPlaylist();
    void savePlaylist();

    void readConfig();
    void writeConfig();

    void readPlaylistSectionOrder();
    void writePlaylistSectionOrder();

    void readPopupMenusSettings();
    void writePopupMenusSettings();

private:
    QPEApplication     *application;

    emuEngine           myEmuEngine;
    struct emuConfig    myEmuConfig;

    uword               selectedSong;
    uword               totalSongs;

    sidTune            *pTune;
    struct sidTuneInfo  myTuneInfo;
    audioDriver         myAudio;
    int                 bufSize;
    ubyte              *pBuffer;

    long                listeningMileage;

    QTimer              playbackTimer;
    QTimer              volumeTimer;
    bool                increaseVolume;
    QTimer              shutdownTimer;

    QButtonGroup       *stateBG;
    QButtonGroup       *speedBG;
    QButtonGroup       *memoryModeBG;
    QButtonGroup       *clockSpeedBG;

    int vol1,  vol2,  vol3,  vol4;
    int pan1L, pan2L, pan3L, pan4L;
    int pan1R, pan2R, pan3R, pan4R;

    int vol1save, vol2save, vol3save, vol4save;

    bool allowPopulate;
    QString currentDir;
    int pathPixelWidth;

    bool audioDeviceIsOpen;
    int frgs; // used to set device specific
    int szbs; // audio buffer sizes

    QPopupMenu *FileMenu;
    int showParentDirEntryID, showHiddenEntriesID, homeButtonTogglesViewID,
        disableHWButtonsID;

    QPopupMenu *PlaylistMenu;
    int allowSortingID, allowMovingID, consistentFileViewID;
    bool playlistChanged;
    int playlistSortingColumn;

    PopupMenuEventFilter *pmef;
    TimeDisplayEventFilter *tdef;

    QLabel *SplashLabel;
};


class ItemBase : public QListViewItem
{
public:
    bool isOdd;
    static float blend; // 1. ... base (white), 0. ... background (green)
    static float dimm;  // helpful for very light background colors

    ItemBase( QListView *parent );
    ItemBase( QListView *parent, QListViewItem *after );

    void paintCell( QPainter *p, const QColorGroup &cg, int column, int width, int alignment )
        {
            if ( isOdd )
            {
                QColorGroup shadedcg = QColorGroup( cg );
                QColor a = cg.background();
                QColor b = cg.base();
                QColor c = QColor( ( a.red()   + ( b.red()   - a.red()   ) * blend ) * dimm,
                                   ( a.green() + ( b.green() - a.green() ) * blend ) * dimm,
                                   ( a.blue()  + ( b.blue()  - a.blue()  ) * blend ) * dimm );
                shadedcg.setColor( QColorGroup::Base, c );

                QListViewItem::paintCell( p, shadedcg, column, width, alignment );
            }
            else
                QListViewItem::paintCell( p, cg, column, width, alignment );
        }
};


class FileItem : public ItemBase
{
public:
    FileItem( QListView *parent, const QFileInfo &fi, int type );

    QString key( int column, bool ascending ) const;
    QString getFilePath(){ return fileInfo.filePath(); } // to retain consistent file view
    QString getAbsFilePath(){ return fileInfo.absFilePath(); } // to prevent relpath-growing
    QString getFileName(){ return fileInfo.fileName(); }
    bool    isDir(){ return fileInfo.isDir(); }

private:
    QFileInfo fileInfo;
};


class PlaylistItem : public ItemBase
{
public:
    PlaylistItem( QListView *parent, QListViewItem *after, const QFileInfo &fi,
                  const QString &name,
                  const QString &author,
                  const QString &copyright,
                  const QString &songs );

    QString key( int column, bool ascending ) const;
    QString getFilePath(){ return fileInfo.filePath(); }
    QString getFileName(){ return fileInfo.fileName(); }

private:
    QFileInfo fileInfo;
};


class PopupMenuEventFilter : public QObject
{
public:
    bool eventFilter( QObject *o, QEvent *e )
        { // menu button on     Zaurus                           iPAQ (Opie)
            if ( ( e->type() == QEvent::KeyPress || e->type() == QEvent::Accel ) &&
                 ( (QKeyEvent *) e )->key() == Key_F11 )
            {
                ( (QPopupMenu *) o )->hide();
                return true; // eat event
            }
            return false; // standard event processing
        }
};


class TimeDisplayEventFilter : public QObject
{
    Q_OBJECT

public:
    bool eventFilter( QObject *, QEvent *e )
        {
            if ( e->type() == QEvent::MouseButtonPress )
            {
                emit mousePressed();
                return true; // eat event
            }
            if ( e->type() == QEvent::MouseButtonRelease )
            {
                emit mouseReleased();
                return true; // eat event
            }
            return false; // standard event processing
        }

signals:
    void mousePressed();
    void mouseReleased();
};

#endif
