/**************************************************************************************94x78**
**
** This file may be distributed and/or modified under the terms of the GNU General Public
** License version 2 as published by the Free Software Foundation and appearing in the file
** LICENSE.GPL included in the packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE WARRANTY OF DESIGN,
** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
*********************************************************************************************/
#include "sidplayer.h"

#include <qtoolbutton.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qslider.h>

#include <qpe/config.h>


void Sidplayer::sidFilterToggled( bool on )
{
    myEmuConfig.emulateFilter = on;
    myEmuEngine.setConfig( myEmuConfig );
}

void Sidplayer::measuredVolumeToggled( bool on )
{
    myEmuConfig.measuredVolume = on;
    myEmuEngine.setConfig( myEmuConfig );
}

void Sidplayer::mos8580Toggled( bool on )
{
    myEmuConfig.mos8580 = on;
    myEmuEngine.setConfig( myEmuConfig );
}

void Sidplayer::forceSongSpeedToggled( bool on )
{
    myEmuConfig.forceSongSpeed = on;
    //ClockSpeedBox->setEnabled( on );
    myEmuEngine.setConfig( myEmuConfig );
}

void Sidplayer::stereoSurroundToggled( bool on )
{
    if ( on )
        myEmuConfig.volumeControl = SIDEMU_STEREOSURROUND;
    else
        myEmuConfig.volumeControl = SIDEMU_FULLPANNING;
    myEmuEngine.setConfig( myEmuConfig );

    // because LIBSIDPLAY changes the pan position when volumeControl is changed
    pan1Changed( Pan1->value() ); pan2Changed( Pan2->value() );
    pan3Changed( Pan3->value() ); pan4Changed( Pan4->value() );
}

void Sidplayer::autoPanningToggled( bool on )
{
    static int p1, p2, p3, p4;

    if ( on )
    {
        myEmuConfig.autoPanning = SIDEMU_CENTEREDAUTOPANNING;
        p1 = Pan1->value(); p2 = Pan2->value();    Pan1->setValue( 0 ); Pan2->setValue( 511 );
        p3 = Pan3->value(); p4 = Pan4->value();    Pan3->setValue( 0 ); Pan4->setValue( 255 );
    }
    else
    {
        myEmuConfig.autoPanning = SIDEMU_NONE;
        Pan1->setValue( p1 ); Pan2->setValue( p2 );
        Pan3->setValue( p3 ); Pan4->setValue( p4 );
    }
    myEmuEngine.setConfig( myEmuConfig );

    // because LIBSIDPLAY changes the pan position when autoPanning is changed
    pan1Changed( Pan1->value() ); pan2Changed( Pan2->value() );
    pan3Changed( Pan3->value() ); pan4Changed( Pan4->value() );
}

//--------------------------------------------------------------------------------------------

void Sidplayer::bankSwitchingToggled( bool on )
{
    if ( on )
        myEmuConfig.memoryMode = MPU_BANK_SWITCHING;
    myEmuEngine.setConfig( myEmuConfig );
}

void Sidplayer::transparentROMToggled( bool on )
{
    if ( on )
        myEmuConfig.memoryMode = MPU_TRANSPARENT_ROM;
    myEmuEngine.setConfig( myEmuConfig );
}

void Sidplayer::playSidEnvToggled( bool on )
{
    if ( on )
        myEmuConfig.memoryMode = MPU_PLAYSID_ENVIRONMENT;
    myEmuEngine.setConfig( myEmuConfig );
}

//--------------------------------------------------------------------------------------------

void Sidplayer::palToggled( bool on )
{
    if ( on )
        myEmuConfig.clockSpeed = SIDTUNE_CLOCK_PAL;
    myEmuEngine.setConfig( myEmuConfig );
}

void Sidplayer::ntscToggled( bool on )
{
    if ( on )
        myEmuConfig.clockSpeed = SIDTUNE_CLOCK_NTSC;
    myEmuEngine.setConfig( myEmuConfig );
}

//--------------------------------------------------------------------------------------------

void Sidplayer::engineDefaultsClicked()
{
    if ( !SIDFilter->isChecked() )
        SIDFilter->setChecked( true );
    if ( !MeasuredVolume->isChecked() )
        MeasuredVolume->setChecked( true );
    if ( MOS8580->isChecked() )
        MOS8580->setChecked( false );
    if ( ForceSongSpeed->isChecked() )
        ForceSongSpeed->setChecked( false );
    if ( StereoSurround->isChecked() )
        StereoSurround->setChecked( false );
    if ( AutoPanning->isChecked() )
        AutoPanning->setChecked( false );

    if ( !BankSwitching->isChecked() )
        BankSwitching->setChecked( true );

    if ( !PAL->isChecked() )
        PAL->setChecked( true );
}

//--------------------------------------------------------------------------------------------

void Sidplayer::readConfig()
{
    Config cfg("Sidplayer");
    cfg.setGroup("Engine");

    SIDFilter->setChecked(      cfg.readBoolEntry( "emulateFilter",  true  ) );
    MeasuredVolume->setChecked( cfg.readBoolEntry( "measuredVolume", true  ) );
    MOS8580->setChecked(        cfg.readBoolEntry( "mos8580",        false ) );
    ForceSongSpeed->setChecked( cfg.readBoolEntry( "forceSongSpeed", false ) );
    StereoSurround->setChecked( cfg.readBoolEntry( "stereoSurround", false ) );
    AutoPanning->setChecked(    cfg.readBoolEntry( "autoPanning",    false ) );

    switch ( cfg.readNumEntry( "memoryMode", 0 ) )
    {
    case MPU_BANK_SWITCHING:
        BankSwitching->setChecked( true );
        break;
    case MPU_TRANSPARENT_ROM:
        TransparentROM->setChecked( true );
        break;
    case MPU_PLAYSID_ENVIRONMENT:
        PlaySidEnv->setChecked( true );
        break;
    default:
        BankSwitching->setChecked( true );
        break;
    }

    switch ( cfg.readNumEntry( "clockSpeed", 0 ) )
    {
    case SIDTUNE_CLOCK_PAL:
        PAL->setChecked( true );
        break;
    case SIDTUNE_CLOCK_NTSC:
        NTSC->setChecked( true );
        break;
    default:
        PAL->setChecked( true );
        break;
    }

    cfg.setGroup("Audio");

    ReleaseAudioDevice->setChecked( cfg.readBoolEntry( "releaseAudioDevice", false ) );
}

void Sidplayer::writeConfig()
{
    Config cfg("Sidplayer");
    cfg.setGroup("Engine");

    if ( !cfg.hasKey( "emulateFilter" ) ||
         cfg.readBoolEntry( "emulateFilter" ) != SIDFilter->isChecked() )
        cfg.writeEntry( "emulateFilter", SIDFilter->isChecked() );
    if ( !cfg.hasKey( "measuredVolume" ) ||
         cfg.readBoolEntry( "measuredVolume" ) != MeasuredVolume->isChecked() )
        cfg.writeEntry( "measuredVolume", MeasuredVolume->isChecked() );
    if ( !cfg.hasKey( "mos8580" ) ||
         cfg.readBoolEntry( "mos8580" ) != MOS8580->isChecked() )
        cfg.writeEntry( "mos8580", MOS8580->isChecked() );
    if ( !cfg.hasKey( "forceSongSpeed" ) ||
         cfg.readBoolEntry( "forceSongSpeed" ) != ForceSongSpeed->isChecked() )
        cfg.writeEntry( "forceSongSpeed", ForceSongSpeed->isChecked() );
    if ( !cfg.hasKey( "stereoSurround" ) ||
         cfg.readBoolEntry( "stereoSurround" ) != StereoSurround->isChecked() )
        cfg.writeEntry( "stereoSurround", StereoSurround->isChecked() );
    if ( !cfg.hasKey( "autoPanning" ) ||
         cfg.readBoolEntry( "autoPanning" ) != AutoPanning->isChecked() )
        cfg.writeEntry( "autoPanning", AutoPanning->isChecked() );

    if ( !cfg.hasKey( "memoryMode" ) ||
         cfg.readNumEntry( "memoryMode" ) != myEmuConfig.memoryMode )
        cfg.writeEntry( "memoryMode", myEmuConfig.memoryMode );

    if ( !cfg.hasKey( "clockSpeed" ) ||
         cfg.readNumEntry( "clockSpeed" ) != myEmuConfig.clockSpeed )
        cfg.writeEntry( "clockSpeed", myEmuConfig.clockSpeed );

    cfg.setGroup("Audio");

    if ( !cfg.hasKey( "releaseAudioDevice" ) ||
         cfg.readBoolEntry( "releaseAudioDevice" ) != ReleaseAudioDevice->isChecked() )
        cfg.writeEntry( "releaseAudioDevice", ReleaseAudioDevice->isChecked() );
}

//--------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------

void Sidplayer::releaseAudioDeviceToggled( bool )
{
    if ( StopButton->isOn() )
        if ( ReleaseAudioDevice->isChecked() )
            closeAudio();
        else
            openAudio();
}
