/**************************************************************************************94x78**
**
** This file may be distributed and/or modified under the terms of the GNU General Public
** License version 2 as published by the Free Software Foundation and appearing in the file
** LICENSE.GPL included in the packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE WARRANTY OF DESIGN,
** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
*********************************************************************************************/
#include "sidplayer.h"
#include "volumebarwidget.h"

#include <qtoolbutton.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <qradiobutton.h>
#include <qcheckbox.h>
#include <qtabwidget.h>

#include <qpe/config.h>
#ifdef Q_WS_QWS
#include <qpe/qcopenvelope_qws.h>
#endif


// bool QWidget::event ( QEvent * e ) [virtual protected]
//
// This is the main event handler. You may reimplement this function in a subclass, but we
// recommend using one of the specialized event handlers instead.
//
// The main event handler first passes an event through all event filters that have been
// installed. If none of the filters intercept the event, it calls one of the specialized
// event handlers.

bool Sidplayer::eventFilter( QObject *o, QEvent *e ) // dedicated (magic) function name
{
    //cout << e->type() << endl;

    if ( FileMenu->isItemChecked( disableHWButtonsID ) &&
         ( e->type() == QEvent::KeyPress ||
           e->type() == QEvent::KeyRelease ||
           e->type() == QEvent::Accel ) )
        return true; // eat event

    static bool processing = false;
    // This is necessary to prevent that this function gets reentered while
    // still in the called function fileViewPressed().  In that function I do
    // some tricky processEvents(), which leads to a crash when the events
    // occure to fast (key auto-repeat).
    //
    // It won't be necessary if I did't mess around with processEvents().
    if ( processing )
        return true; // doesn't matter whether the events are eaten or not

    processing = true;
    if ( e->type() == QEvent::KeyPress )
    {
        QKeyEvent *k = (QKeyEvent *) e;
        switch ( k->key() )
        {
        case Key_Return:
        case Key_Enter:
        {
            if ( o == FileView )
                fileViewPressed( FileView->currentItem() );
            else if ( o == PlaylistView )
                playlistViewPressed( PlaylistView->currentItem() );
            processing = false;
            return true; // eat event
            break;
        }
        case Key_Space:
        {
            Speed10x->setChecked( true );
            processing = false;
            return true; // eat event
            break;
        }
        case Key_Left:
        {
            if ( SongNumber->value() > 1 )
                SongNumber->stepDown();
            processing = false;
            return true; // eat event
            break;
        }
        case Key_Right:
        {
            if ( SongNumber->value() < totalSongs )
                SongNumber->stepUp();
            processing = false;
            return true; // eat event
            break;
        }
        case Key_Up:
        {
            if ( o == FileView )
            {
                FileItem *above = (FileItem *) FileView->currentItem()->itemAbove();
                if ( above )
                    if ( !above->isDir() )
                        fileViewPressed( above );
                    else
                    {
                        FileView->ensureItemVisible( above->itemAbove() );
                        FileView->ensureItemVisible( above );
                        FileView->setCurrentItem( above ); // Focus
                        FileView->setSelected( above, true );
                    }
            }
            else if ( o == PlaylistView )
            {
                prevButtonClicked();
            }
            processing = false;
            return true; // eat event
            break;
        }
        case Key_Down:
        {
            if ( o == FileView )
            {
                FileItem *below = (FileItem *) FileView->currentItem()->itemBelow();
                if ( below )
                    if ( !below->isDir() )
                        fileViewPressed( below );
                    else
                    {
                        FileView->ensureItemVisible( below->itemBelow() );
                        FileView->ensureItemVisible( below );
                        FileView->setCurrentItem( below ); // Focus
                        FileView->setSelected( below, true );
                    }
            }
            else if ( o == PlaylistView )
            {
                nextButtonClicked();
            }
            processing = false;
            return true; // eat event
            break;
        }
        case Key_Escape:
        //case Key_Delete: // for testing purpose
        {
            shutdownTimer.stop();
            if ( PauseButton->isEnabled() && PlayButton->isOn() )
                PauseButton->setOn( true );
            else if ( PlayButton->isEnabled() && PauseButton->isOn() )
                PlayButton->setOn( true );
            processing = false;
            return true; // eat event
            break;
        }
        case Key_F9:  // to prevent starting the application associated with the buttons
        {             // it was necessary to call QPEApplication::grabKeyboard(); in main.cpp
            increaseVolume = false;
            volumeTimerTimeout(); // Call it one time manually, otherwise it wont get
                                  // called at all when a button is pressed for a time
                                  // shorter than VOLUME_TIMER_INTERVAL.
            volumeTimer.start( VOLUME_TIMER_INTERVAL, false );
            processing = false;
            return true; // eat event
            break;
        }
        case Key_F10:
        {
            increaseVolume = true;
            volumeTimerTimeout(); // Call it one time manually, otherwise it wont get
                                  // called at all when a button is pressed for a time
                                  // shorter than VOLUME_TIMER_INTERVAL.
            volumeTimer.start( VOLUME_TIMER_INTERVAL, false );
            processing = false;
            return true; // eat event
            break;
        }
        case Key_F11: // menu button on my Qtopia-Zaurus
        {
            if ( o == FileView )
            {
                if ( !FileMenuButton->isDown() )
                {
                    FileMenuButton->setDown( true );
                    fileMenuButtonPressed();
                }
            }
            else if ( o == PlaylistView )
            {
                if ( !ListMenuButton->isDown() )
                {
                    ListMenuButton->setDown( true );
                    listMenuButtonPressed();
                }
            }
            processing = false;
            return true; // eat event
            break;
        }
        case Key_F12: // Home-Key
        {
            if ( FileMenu->isItemChecked( homeButtonTogglesViewID ) )
            {
                //TabWidget->setCurrentPage( ( TabWidget->currentPageIndex() + 1 ) % 5 );
                if ( TabWidget->currentPageIndex() == 0 )
                    TabWidget->setCurrentPage( 1 );
                else
                    TabWidget->setCurrentPage( 0 );
            }
            else
            {
                // Works with QPE & Qtopia in older versions
                // but confuses newer vesions when "Fast load" is enabled.
                //QPEApplication::ungrabKeyboard();
                //hide();

                // Only works with newer versions of Qtopia & Opie :(
#if defined(Q_WS_QWS) && !defined(QT_NO_COP)
                QCopEnvelope ( "QPE/Application/qpe", "raise()" ); // to desktop (home)
                //QCopEnvelope ( "QPE/Launcher", "home()" ); // seems to do the same
#endif
            }
            processing = false;
            return true; // eat event
            break;
        }
        case Key_F13: // Zaurus Email-Key
        {
            if ( o == FileView )
                goToParentDir();
            processing = false;
            return true; // eat event
            break;
        }
        default:
            break;
        }
    }
    else if ( e->type() == QEvent::Accel ) // menu button on iPAQ (Opie)
    {                                      // (boy, I *hate* these platform differences)
        QKeyEvent *k = (QKeyEvent *) e;
        switch ( k->key() )
        {
        case Key_F11:
        {
            if ( TabWidget->currentPageIndex() == 0 ) // oh boy!  "o" doesn't seem to be
            {                                         // either FileView or PlaylistView
                if ( !FileMenuButton->isDown() )      // when the event is of type Accel
                {
                    FileMenuButton->setDown( true );
                    fileMenuButtonPressed();
                }
            }
            else if ( TabWidget->currentPageIndex() == 1 )
            {
                if ( !ListMenuButton->isDown() )
                {
                    ListMenuButton->setDown( true );
                    listMenuButtonPressed();
                }
            }
            processing = false;
            return true; // eat event
            break;
        }
        default:
            break;
        }
    }
    else if ( e->type() == QEvent::KeyRelease )
    {
        QKeyEvent *k = (QKeyEvent *) e;
        switch ( k->key() )
        {
        case Key_Space:
        {
            Speed1x->setChecked( true ); // for visual feedback only
            processing = false;
            return true; // eat event
            break;
        }
        case Key_F9:
        case Key_F10:
        {
            volumeTimer.stop();
            processing = false;
            return true; // eat event
            break;
        }
        default:
            break;
        }
    }
/* is now dealt with in closeEvent()
    else if ( e->type() == QEvent::Close ) // Workaround because of an ugly hack in
    {                                      // QPEApplication::qwsEventFilter() which sends a
        if ( !StopButton->isOn() )         // close event when the Key_Escape key is pressed,
        {                                  // regardless whether the key event is consumed by
            processing = false;            // an installed eventFilter() or not.
            return true; // eat event      // Unfortunately this workaround prevents the user
        }                                  // from closing this application unless the
    }                                      // StopButton->isOn().
*/
    processing = false;
    return false; // standard event processing
    //return QWidget::eventFilter( o, e ); // ??
}

//--------------------------------------------------------------------------------------------

void Sidplayer::showEvent( QShowEvent * ) // dedicated (magic) function name
{
    fitFileViewColumnWidths();

    if ( SplashLabel )
    {
        SplashLabel->setFixedSize( file->width(),
                                   LowerLine->y() - UpperLine->y() - UpperLine->height() );
        SplashLabel->move( 0, UpperLine->y() + UpperLine->height() );
    }

    QPEApplication::grabKeyboard(); // Grab the keyboard such that the system's application
                                    // launching keys no longer work, and instead are
                                    // receivable by this application.

    if ( !audioDeviceIsOpen && !ReleaseAudioDevice->isChecked() )
        openAudio();

    VolumeBar->getVolume();
}

void Sidplayer::resizeEvent( QResizeEvent * ) // dedicated (magic) function name
{
    fitFileViewColumnWidths();

    //                  to get rid of the QWidget::setMin/MaximumSize warnings during startup
    if ( SplashLabel && isVisible() )
    {
        SplashLabel->setFixedSize( file->width(),
                                   LowerLine->y() - UpperLine->y() - UpperLine->height() );
        SplashLabel->move( 0, UpperLine->y() + UpperLine->height() );
    }
}

void Sidplayer::shutdownTimerTimeout()
{
    StopButton->setOn( true );
    close();
}

void Sidplayer::closeEvent( QCloseEvent *e ) // dedicated (magic) function name
{
    // workaround - see the comment at the end of eventFilter()
    if ( !StopButton->isOn() )
    {
        shutdownTimer.start( SHUTDOWN_TIMER_TIMEOUT, true );
        return;
    }

    e->accept();

    {
    // moved the following stuff from the destructor here in case of activated fastload
    // and system crash

    if ( playlistChanged )
        savePlaylist();

    writeConfig();
    writePlaylistSectionOrder();
    writePopupMenusSettings();

    Config cfg("Sidplayer");
    cfg.setGroup("List");

    if ( !cfg.hasKey( "sortingColumn" ) ||
         cfg.readEntry( "sortingColumn" ) != playlistSortingColumn )
        cfg.writeEntry( "sortingColumn", playlistSortingColumn );

    QString selectedEntry = "";
    if ( PlaylistView->selectedItem() )
        selectedEntry = ( (PlaylistItem *) PlaylistView->selectedItem() )->getFilePath();
    if ( !cfg.hasKey( "selectedEntry" ) || cfg.readEntry( "selectedEntry" ) != selectedEntry )
        cfg.writeEntry( "selectedEntry", selectedEntry );

    cfg.setGroup("Path");

    if ( !cfg.hasKey( "currentDir" ) || cfg.readEntry( "currentDir" ) != currentDir )
        cfg.writeEntry( "currentDir", currentDir );

    selectedEntry = "";
    if ( FileView->selectedItem() )
        selectedEntry = ( (FileItem *) FileView->selectedItem() )->getFilePath();
    if ( !cfg.hasKey( "selectedEntry" ) || cfg.readEntry( "selectedEntry" ) != selectedEntry )
        cfg.writeEntry( "selectedEntry", selectedEntry );

    cfg.setGroup("Various");
    int secondsTotal = myEmuEngine.getSecondsTotal();
    if ( secondsTotal != 0 || !cfg.hasKey( "listeningMileage" ) )
        cfg.writeEntry( "listeningMileage",
                        QString::number( listeningMileage + secondsTotal ) );

    // end of the stuff which was formerly located in the destructor
    }

    // harmonize VolumeBar and System-Volume

    // Qtopia & Opie in newer versions
    Config cfg( "qpe" );
    cfg.setGroup ( "Volume" );
    QString volumeEntryKey = "VolumePercent";
    if ( ! cfg.hasKey( volumeEntryKey ) )
    {
        // QPE & Qtopia in older versions
        cfg = Config( "Sound" );
        cfg.setGroup( "System" );
        volumeEntryKey = "Volume";
    }

    if ( cfg.readNumEntry( volumeEntryKey ) != VolumeBar->getVolume() )
        cfg.writeEntry( volumeEntryKey, VolumeBar->getVolume() );

#if ( defined Q_WS_QWS || defined(_WS_QWS_) ) && !defined(QT_NO_COP)
    // Send notification that the volume has changed
    QCopEnvelope( "QPE/System", "volumeChange(bool)" ) << false; // false means "not muted"
#endif

    // release audio device
    closeAudio();
}
