/**************************************************************************************94x78**
**
** This file may be distributed and/or modified under the terms of the GNU General Public
** License version 2 as published by the Free Software Foundation and appearing in the file
** LICENSE.GPL included in the packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE WARRANTY OF DESIGN,
** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
*********************************************************************************************/
#include "sidplayer.h"
#include "volumebarwidget.h"

#include <qtoolbutton.h>
#include <qlcdnumber.h>
#include <qlabel.h>
#include <qcheckbox.h>
#include <qdir.h>

#include <qpe/resource.h>
#include <qpe/config.h>
#ifdef Q_WS_QWS
#include <qpe/qcopenvelope_qws.h>
#endif


static bool evenFlag; // This global flag is used to toggle between even and odd list entries
                      // when generating list items (i.e. when populating a list).

FileItem::FileItem( QListView *parent, const QFileInfo &fi, int type )
    : ItemBase( parent ),
      fileInfo( fi )
{
    switch ( type )
    {
    case 0: // Link to Dir
    {
        setText( 0, fi.fileName() + " -> " + fi.readLink() );
	if ( QDir( fi.filePath() ).isReadable() )
	    setPixmap( 0, Resource::loadPixmap( "sidplayer/mime/folder_link" ) );
	else
	    setPixmap( 0, Resource::loadPixmap( "sidplayer/mime/folder_locked_link" ) );
        break;
    }
    case 1: // Dir
    {
        setText( 0, fi.fileName() );
	if ( QDir( fi.filePath() ).isReadable() )
	    setPixmap( 0, Resource::loadPixmap( "sidplayer/mime/folder" ) );
	else
	    setPixmap( 0, Resource::loadPixmap( "sidplayer/mime/folder_locked" ) );
        break;
    }
    case 2: // Link to Tune
    {
        setText( 0, fi.fileName() + " -> " + fi.readLink() );
        if ( fi.isReadable() )
            setPixmap( 0, Resource::loadPixmap( "sidplayer/mime/tune_link" ) );
        else
            setPixmap( 0, Resource::loadPixmap( "sidplayer/mime/tune_locked_link" ) );
        break;
    }
    case 3: // Tune
    {
        setText( 0, fi.fileName() );
        if ( fi.isReadable() )
            setPixmap( 0, Resource::loadPixmap( "sidplayer/mime/tune" ) );
        else
            setPixmap( 0, Resource::loadPixmap( "sidplayer/mime/tune_locked" ) );
        break;
    }
    case 4: // Parent Dir
    {
        setText( 0, fi.fileName() );
	if ( QDir( fi.filePath() ).isReadable() )
	    setPixmap( 0, Resource::loadPixmap( "sidplayer/mime/parentdir" ) );
	else
	    setPixmap( 0, Resource::loadPixmap( "sidplayer/mime/parentdir_locked" ) );
        break;
    }
    default:
        break;
    }

    isOdd = evenFlag = !evenFlag;
}

QString FileItem::key( int column, bool ascending ) const
{
    ascending = ascending; // use the parameter to satisfy the compiler

    if ( fileInfo.isDir() )
    {
        // We want the directories to appear at the top of the list
        return ( (char) 0 + text(column).lower() );
    }

    return text(column).lower();
}

//============================================================================================
//============================================================================================

void Sidplayer::setAllFileViewItemsOddFlag()
{
    FileItem *fi = (FileItem *) FileView->firstChild();
    if ( fi )
    {
        fi->isOdd = false;
        setFollowingItemsOddFlag( fi );
    }
}

//--------------------------------------------------------------------------------------------

void Sidplayer::fileMenuButtonPressed()
{
    QPoint pos = FileMenuButton->mapToGlobal( QPoint(0,0) );
    int x = pos.x();
    int y = pos.y() + FileMenuButton->size().height();
    FileMenu->popup( QPoint( x, y ) );
}

void Sidplayer::onFileMenuHide()
{
    FileMenuButton->setDown( false );
}

//--------------------------------------------------------------------------------------------

void Sidplayer::reloadFileView()
{
    QString selectThis = "";
    if ( FileView->selectedItem() )
        selectThis = ( (FileItem *) FileView->selectedItem() )->getFilePath();

    populateFileView( currentDir, selectThis );
}

void Sidplayer::deleteFile()
{
    FileItem *fileItem = (FileItem *) FileView->selectedItem();
    if ( fileItem == NULL )
        return;

    QFileInfo fi = QFileInfo( fileItem->getFilePath() );
    if ( fi.isDir() )
        return;

    QFile f;
    f.setName( fileItem->getFilePath() );

    if ( f.remove() )
    {
        FileView->takeItem( fileItem );

        fileItem = (FileItem *) FileView->currentItem();
        if ( fileItem )
        {
            fileItem->isOdd = !fileItem->isOdd;
            setFollowingItemsOddFlag( fileItem );
        }

        // necessary in the case the reduced amount of files does not need a scrollbar
        application->processEvents(); // to get the new FileView->viewport()->width()
        FileView->setColumnWidth( 0, FileView->viewport()->width() );
    }
}

void Sidplayer::showParentDirEntry()
{
    FileMenu->setItemChecked( showParentDirEntryID,
                              !FileMenu->isItemChecked( showParentDirEntryID ) );
    reloadFileView();
}

void Sidplayer::showHiddenEntries()
{
    FileMenu->setItemChecked( showHiddenEntriesID,
                              !FileMenu->isItemChecked( showHiddenEntriesID ) );
    reloadFileView();
}

void Sidplayer::disableHWButtons()
{
    FileMenu->setItemChecked( disableHWButtonsID,
                              !FileMenu->isItemChecked( disableHWButtonsID ) );
}

void Sidplayer::homeButtonTogglesView()
{
    FileMenu->setItemChecked( homeButtonTogglesViewID,
                              !FileMenu->isItemChecked( homeButtonTogglesViewID ) );
}

//--------------------------------------------------------------------------------------------

void Sidplayer::parentDirButtonPressed()
{
    goToParentDir();
}

void Sidplayer::goToParentDir()
{
    populateFileView( QDir::cleanDirPath( currentDir + "/.." ),
                      currentDir );
}

// almost identical to Sidplayer::selectPlaylistItem()
void Sidplayer::selectFileItem( const QString &selectThis )
{
    // find the item in the filelist and select it
    QListViewItemIterator it( FileView );
    for ( ; it.current(); ++it )
    {
        FileItem *item = (FileItem *) it.current();
        if ( item->getFilePath() == selectThis ) // relative path (important)
        {
            FileView->ensureItemVisible( item->itemAbove() );
            FileView->ensureItemVisible( item->itemBelow() );
            FileView->ensureItemVisible( item );
            FileView->setCurrentItem( item ); // Focus
            FileView->setSelected( item, true );
            return;
        }
    }
    // not found
    FileItem *item = (FileItem *) FileView->firstChild();
    FileView->ensureItemVisible( item ); // scroll to the top
    FileView->setCurrentItem( item ); // highlight the first item
    FileView->setSelected( item, false ); // but don't select it
}

void Sidplayer::fitFileViewColumnWidths()
{
    FileView->setColumnWidth( 0, FileView->viewport()->width() );
    FileView->triggerUpdate(); // I have no idea why this is needed, but it *is* needed.

    //if ( pathPixelWidth <= FileView->viewport()->width() )
    if ( pathPixelWidth <= EllipsisLabel->width() + PathLabel->width() - 1 )
    {
        EllipsisLabel->setText( "" );
        PathLabel->setAlignment( AlignLeft | AlignVCenter | ExpandTabs );
    }
    else
    {
        EllipsisLabel->setText( "..." );
        PathLabel->setAlignment( AlignRight | AlignVCenter | ExpandTabs );
    }
}

void Sidplayer::populateFileView( const QString &directory, const QString &selectThis = "" )
{
    QDir dir( directory );

    if ( !dir.exists() || !dir.isReadable() )
        return;

    QString absPath = dir.absPath(); // symbolic links in the path are preserved
    QFontMetrics fm = QFontMetrics( PathLabel->font() );
    pathPixelWidth = fm.width( absPath ); // used in fitFileViewColumnWidths()
    PathLabel->setText( absPath );

    if ( FileMenu->isItemChecked( showHiddenEntriesID ) )
        dir.setFilter( QDir::Dirs | QDir::Files | QDir::Hidden ); //| QDir::NoSymLinks );
    else
        dir.setFilter( QDir::Dirs | QDir::Files ); //| QDir::NoSymLinks );

    const QFileInfoList *li = dir.entryInfoList();
    QFileInfoListIterator it( *li );
    QFileInfo *fi;

    if ( QDir( directory + "/.." ).isReadable() )
        ParentDirButton->setIconSet( loadIconSet( "sidplayer/mime/parentdir" ) );
    else
        ParentDirButton->setIconSet( loadIconSet( "sidplayer/mime/parentdir_locked" ) );
    ParentDirButton->setEnabled( directory != "/" );

    FileView->clear();
    if ( directory != "/" && // relies on absolute path
         FileMenu->isItemChecked( showParentDirEntryID ) )
        (void) new FileItem( FileView, QFileInfo( directory + "/.." ), 4 );
    while ( (fi = it.current()) )
    {
        if ( fi->isDir() )
        {
            if ( fi->fileName() == "." || fi->fileName() == ".." )
                ;
            else if ( fi->isSymLink() )
                (void) new FileItem( FileView, *fi, 0 );
            else
                (void) new FileItem( FileView, *fi, 1 );
        }
        else if ( fi->extension( false ).lower() == "sid" ||
                  fi->extension( false ).lower() == "psid" ||
                  fi->extension( false ).lower() == "dat" )
            if ( fi->isSymLink() )
                (void) new FileItem( FileView, *fi, 2 );
            else
                (void) new FileItem( FileView, *fi, 3 );
        ++it;
    }

    if ( selectThis == "" )
    {
        FileItem *item = (FileItem *) FileView->firstChild();
        FileView->ensureItemVisible( item ); // scroll to the top
        FileView->setCurrentItem( item ); // highlight the first item
        FileView->setSelected( item, false ); // but don't select it
    }
    else
        selectFileItem( selectThis );

    setAllFileViewItemsOddFlag(); // Necessary because directories are sortet as the first
                                  // elements in the list.
    fitFileViewColumnWidths();

    currentDir = directory;
}

//--------------------------------------------------------------------------------------------

void Sidplayer::fileViewPressed( QListViewItem *item )
{
    if ( item == NULL )
        return;

    FileView->setSelected( item, true );
    FileView->ensureItemVisible( item->itemAbove() );
    FileView->ensureItemVisible( item->itemBelow() );
    FileView->ensureItemVisible( item );
    application->processOneEvent(); // give immediate visual feedback

    FileItem *fi = (FileItem *) item;
    if ( fi->getFileName() == ".." )
        goToParentDir();
    else if ( fi->isDir() )
        populateFileView( fi->getAbsFilePath() ); // absolute path (important)
    else
    {
        allowPopulate = false;
        setDocument( fi->getFilePath() ); // abs or rel (not important)
    }
}

//--------------------------------------------------------------------------------------------

void Sidplayer::stopButtonToggled( bool on )
{
    if ( on )
    {
        stop();
        Time->display( " 0:00" );
        if ( ReleaseAudioDevice->isChecked() )
            closeAudio();
        PauseButton->setEnabled( false );
        StopButton->setEnabled( false );
        PlayButton->setEnabled( true );
    }
    else if ( !PlayButton->isOn() ) // due to not successfully opened audio device
        StopButton->setOn( true );
}

void Sidplayer::pauseButtonToggled( bool on )
{
    if ( on )
    {
        stop();
        StopButton->setEnabled( true );
        PauseButton->setEnabled( false );
        PlayButton->setEnabled( true );
    }
}

void Sidplayer::playButtonToggled( bool on )
{
    if ( on )
    {
        if ( !audioDeviceIsOpen )
            if ( !openAudio() )
            {
                // audio device opening was not successfull
                PlayButton->setOn( false );
                return;
            }
        if ( StopButton->isOn() )
            initializeEngine( selectedSong ); // sets selectedSong, rewinds to the beginning
        setCaptionAccordingToSong(); // uses selectedSong
        StopButton->setEnabled( true );
        PauseButton->setEnabled( true );
        PlayButton->setEnabled( false );
        start();
    }
}

void Sidplayer::songNumberChanged( int number )
{
    initializeEngine( number ); // sets selectedSong, rewinds to the beginning
    if ( PlayButton->isOn() )
        playButtonToggled( true );
    else
        PlayButton->setOn( true );
}

//--------------------------------------------------------------------------------------------

void Sidplayer::speed1xToggled( bool on )
{
    if ( on )
        sidEmuFastForwardReplay( 100 ); // calc 100%
}

void Sidplayer::speed2xToggled( bool on )
{
    if ( on )
        sidEmuFastForwardReplay( 50 ); // calc 50%
}

void Sidplayer::speed4xToggled( bool on )
{
    if ( on )
        sidEmuFastForwardReplay( 25 ); // calc 25%
}

void Sidplayer::speed10xToggled( bool on )
{
    if ( on )
        sidEmuFastForwardReplay( 10 ); // calc 10%
}
