/**************************************************************************************94x78**
**
** This file may be distributed and/or modified under the terms of the GNU General Public
** License version 2 as published by the Free Software Foundation and appearing in the file
** LICENSE.GPL included in the packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE WARRANTY OF DESIGN,
** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
*********************************************************************************************/
#include "sidplayer.h"

#include <time.h>
#include <stdlib.h>

#include <qtoolbutton.h>
#include <qdir.h>
#include <qtextstream.h>
#include <qlabel.h>
#include <qheader.h>

#include <qpe/global.h>
#include <qpe/config.h>


static bool evenFlag; // This global flag is used to toggle between even and odd list entries
                      // when generating list items (i.e. when populating a list).

PlaylistItem::PlaylistItem( QListView *parent, QListViewItem *after, const QFileInfo &fi,
                            const QString &name,
                            const QString &author,
                            const QString &copyright,
                            const QString &songs )
    : ItemBase( parent, after ),
      fileInfo( fi )
{
    setText( 0, name );
    setText( 1, author );
    setText( 2, copyright );
    setText( 3, songs );
    setText( 4, fi.fileName() );

    isOdd = evenFlag = !evenFlag;
}

QString PlaylistItem::key( int column, bool ascending ) const
{
    ascending = ascending; // use the parameter to satisfy the compiler

    return text(column).lower();
}

//============================================================================================
//============================================================================================

void Sidplayer::setAllPlaylistViewItemsOddFlag()
{
    PlaylistItem *pli = (PlaylistItem *) PlaylistView->firstChild();
    if ( pli )
    {
        pli->isOdd = false;
        setFollowingItemsOddFlag( pli );
    }
}

//--------------------------------------------------------------------------------------------

void Sidplayer::listMenuButtonPressed()
{
    QPoint pos = ListMenuButton->mapToGlobal( QPoint(0,0) );
    int x = pos.x();
    int y = pos.y() + ListMenuButton->size().height();
    PlaylistMenu->popup( QPoint( x, y ) );
}

void Sidplayer::onListMenuHide()
{
    ListMenuButton->setDown( false );
}

void Sidplayer::headerClicked( int section )
{
    if ( PlaylistMenu->isItemChecked( allowSortingID ) )
    {
        playlistSortingColumn = section;
        setAllPlaylistViewItemsOddFlag();
        PlaylistView->ensureItemVisible( PlaylistView->currentItem()->itemAbove() );
        PlaylistView->ensureItemVisible( PlaylistView->currentItem()->itemBelow() );
        PlaylistView->ensureItemVisible( PlaylistView->currentItem() );
    }
}

//--------------------------------------------------------------------------------------------

void Sidplayer::insertTune()
{
    FileItem *fileItem = (FileItem *) FileView->currentItem();
    if ( fileItem == NULL )
        return;

    QFileInfo fi = QFileInfo( fileItem->getFilePath() );
    if ( fi.isDir() )
        return;

    PlaylistItem *current = (PlaylistItem *) PlaylistView->currentItem();
    if ( current == NULL )
        evenFlag = true;
    else
        evenFlag = current->isOdd;

    PlaylistView->setSelected( current, false );
    PlaylistView->setSelectionMode( QListView::NoSelection );

    sidTune            localTune = sidTune( fi.filePath() );
    struct sidTuneInfo localTuneInfo;
    localTune.getInfo( localTuneInfo );

    PlaylistItem *pli = new PlaylistItem( PlaylistView, current, fi,
                                          localTuneInfo.nameString,
                                          localTuneInfo.authorString,
                                          localTuneInfo.copyrightString,
                                          QString::number( localTuneInfo.songs ) );

    PlaylistView->setCurrentItem( pli );
    PlaylistView->setSelectionMode( QListView::Single );

    //PlaylistView->setSelected( pli, true );
    PlaylistView->ensureItemVisible( pli->itemAbove() );
    PlaylistView->ensureItemVisible( pli->itemBelow() );
    PlaylistView->ensureItemVisible( pli );

    checkButtonState();
    playlistChanged = true;

    setFollowingItemsOddFlag( pli );
}

void Sidplayer::removeTune()
{
    QListViewItem *item = PlaylistView->selectedItem();
    if ( item == NULL )
        return;

    PlaylistView->takeItem( item );

    checkButtonState();
    playlistChanged = true;

    PlaylistItem *pli = (PlaylistItem *) PlaylistView->currentItem();
    if ( pli )
    {
        pli->isOdd = !pli->isOdd;
        setFollowingItemsOddFlag( pli );
    }
}

void Sidplayer::insertDir()
{
    PlaylistView->setSelected( PlaylistView->currentItem(), false );
    PlaylistView->setSelectionMode( QListView::NoSelection );
    insertDir_();
    PlaylistView->setSelectionMode( QListView::Single );

    //PlaylistView->setSelected( PlaylistView->currentItem(), true );
    PlaylistView->ensureItemVisible( PlaylistView->currentItem()->itemAbove() );
    PlaylistView->ensureItemVisible( PlaylistView->currentItem()->itemBelow() );
    PlaylistView->ensureItemVisible( PlaylistView->currentItem() );

    checkButtonState();
    playlistChanged = true;

    setAllPlaylistViewItemsOddFlag();
}

void Sidplayer::insertDir_( QString dirpath = "" )
{
    if ( dirpath == "" )
        dirpath = currentDir;

    QDir dir( dirpath );
    dir.setFilter( QDir::Files );

    const QFileInfoList *li = dir.entryInfoList();
    QFileInfoListIterator it( *li );
    QFileInfo *fi;

    PlaylistItem *pli = (PlaylistItem *) PlaylistView->currentItem();
    while ( (fi = it.current()) )
    {
        if ( fi->extension( false ).lower() == "sid" ||
             fi->extension( false ).lower() == "psid" ||
             fi->extension( false ).lower() == "dat" )
        {
            sidTune            localTune = sidTune( fi->filePath() );
            struct sidTuneInfo localTuneInfo;
            localTune.getInfo( localTuneInfo );

            pli = new PlaylistItem( PlaylistView, pli, *fi,
                                    localTuneInfo.nameString,
                                    localTuneInfo.authorString,
                                    localTuneInfo.copyrightString,
                                    QString::number( localTuneInfo.songs ) );
        }
        ++it;
    }

    PlaylistView->setCurrentItem( pli );
}

void Sidplayer::insertDirRecursive()
{
    PlaylistView->setSelected( PlaylistView->currentItem(), false );
    PlaylistView->setSelectionMode( QListView::NoSelection );
    insertDirRecursive_();
    PlaylistView->setSelectionMode( QListView::Single );

    //PlaylistView->setSelected( PlaylistView->currentItem(), true );
    PlaylistView->ensureItemVisible( PlaylistView->currentItem()->itemAbove() );
    PlaylistView->ensureItemVisible( PlaylistView->currentItem()->itemBelow() );
    PlaylistView->ensureItemVisible( PlaylistView->currentItem() );

    checkButtonState();
    playlistChanged = true;

    setAllPlaylistViewItemsOddFlag();
}

void Sidplayer::insertDirRecursive_( QString dirpath = "" )
{
    if ( dirpath == "" )
        dirpath = currentDir;

    insertDir_( dirpath );

    QDir dir( dirpath );
    dir.setFilter( QDir::Dirs );

    const QFileInfoList *li = dir.entryInfoList();
    QFileInfoListIterator it( *li );
    QFileInfo *fi;

    while ( (fi = it.current()) )
    {
        if ( fi->fileName() == "." || fi->fileName() == ".." )
            ;
        else
            insertDirRecursive_( fi->filePath() );
        ++it;
    }
}

void Sidplayer::clearList()
{
    PlaylistView->clear();

    checkButtonState();
    playlistChanged = true;
}

void Sidplayer::shuffleList()
{
    if ( PlaylistMenu->isItemChecked( allowSortingID ) )
        allowSorting(); // turn column sorting off

    srand( time( NULL ) );

    long count = PlaylistView->childCount();

    QListViewItemIterator dst( PlaylistView );
    for ( ; dst.current(); ++dst )
    {
        long srcPos = rand() / ( RAND_MAX / count );

        // this
        QListViewItem *srcItem = PlaylistView->firstChild();
        for ( long i = 0; i < srcPos; ++i )
            srcItem = srcItem->nextSibling();

        srcItem->moveItem( dst.current() );

        // is faster than this
        //QListViewItemIterator src( PlaylistView );
        //for ( long i = 0; i < srcPos; ++i )
        //    ++src;
        //
        //src.current()->moveItem( dst.current() );
    }

    PlaylistView->ensureItemVisible( PlaylistView->currentItem()->itemAbove() );
    PlaylistView->ensureItemVisible( PlaylistView->currentItem()->itemBelow() );
    PlaylistView->ensureItemVisible( PlaylistView->currentItem() );

    checkButtonState();
    playlistChanged = true;

    setAllPlaylistViewItemsOddFlag();
}

void Sidplayer::allowSorting()
{
    PlaylistMenu->setItemChecked( allowSortingID,
                                  !PlaylistMenu->isItemChecked( allowSortingID ) );

    if ( PlaylistMenu->isItemChecked( allowSortingID ) )
    {
        playlistSortingColumn = 0;
        PlaylistView->setSorting( playlistSortingColumn );
        PlaylistView->ensureItemVisible( PlaylistView->currentItem()->itemAbove() );
        PlaylistView->ensureItemVisible( PlaylistView->currentItem()->itemBelow() );
        PlaylistView->ensureItemVisible( PlaylistView->currentItem() );
        checkButtonState();
        playlistChanged = true;
        setAllPlaylistViewItemsOddFlag();
    }
    else
    {
        playlistSortingColumn = -1;
        PlaylistView->setSorting( playlistSortingColumn );
    }
}

void Sidplayer::allowMoving()
{
    PlaylistMenu->setItemChecked( allowMovingID,
                                  !PlaylistMenu->isItemChecked( allowMovingID ) );

    checkButtonState();
}

void Sidplayer::consistentFileView()
{
    PlaylistMenu->setItemChecked( consistentFileViewID,
                                  !PlaylistMenu->isItemChecked( consistentFileViewID ) );
}

//--------------------------------------------------------------------------------------------

void Sidplayer::checkButtonState()
{
    long count = PlaylistView->childCount();
    PlaylistItem *ci = (PlaylistItem *) PlaylistView->currentItem();

    TunesLabel->setText( QString::number( count ) );

    if ( count < 2 || ci == NULL )
    {
        PrevButton->setEnabled( false );
        NextButton->setEnabled( false );
        UpButton->setEnabled( false );
        DownButton->setEnabled( false );

        return;
    }

    if ( ci->itemAbove() != NULL )
        PrevButton->setEnabled( true );
    else
        PrevButton->setEnabled( false );

    if ( ci->itemBelow() != NULL )
        NextButton->setEnabled( true );
    else
        NextButton->setEnabled( false );

    PlaylistItem *si = (PlaylistItem *) PlaylistView->selectedItem();
    if ( si == NULL )
    {
        UpButton->setEnabled( false );
        DownButton->setEnabled( false );
        return;
    }

    if ( si->itemAbove() != NULL )
        UpButton->setEnabled( PlaylistMenu->isItemChecked( allowMovingID ) );
    else
        UpButton->setEnabled( false );

    if ( si->itemBelow() != NULL )
        DownButton->setEnabled( PlaylistMenu->isItemChecked( allowMovingID ) );
    else
        DownButton->setEnabled( false );
}

void Sidplayer::playlistViewPressed( QListViewItem *item )
{
    if( item == NULL )
        return;

    PlaylistView->setSelected( item, true );
    PlaylistView->ensureItemVisible( item->itemAbove() );
    PlaylistView->ensureItemVisible( item->itemBelow() );
    PlaylistView->ensureItemVisible( item );
    application->processOneEvent(); // give immediate visual feedback

    PlaylistItem *pli = (PlaylistItem *) item;
    if ( !PlaylistMenu->isItemChecked( consistentFileViewID ) )
        allowPopulate = false;

    QFileInfo fi = QFileInfo( pli->getFilePath() );
    if ( fi.exists() )
        setDocument( pli->getFilePath() );

    checkButtonState();
}

//--------------------------------------------------------------------------------------------

void Sidplayer::prevButtonClicked()
{
    playlistViewPressed( PlaylistView->currentItem()->itemAbove() );
}

void Sidplayer::nextButtonClicked()
{
    playlistViewPressed( PlaylistView->currentItem()->itemBelow() );
}

void Sidplayer::upButtonClicked()
{
    if ( PlaylistMenu->isItemChecked( allowSortingID ) )
        allowSorting(); // turn column sorting off

    PlaylistItem *above = (PlaylistItem *) PlaylistView->selectedItem()->itemAbove();
    PlaylistItem *selected = (PlaylistItem *) PlaylistView->selectedItem();
    above->moveItem( selected );

    PlaylistView->ensureItemVisible( selected->itemAbove() );
    PlaylistView->ensureItemVisible( selected->itemBelow() );
    PlaylistView->ensureItemVisible( selected );

    checkButtonState();
    playlistChanged = true;

    above->isOdd    = !above->isOdd;
    selected->isOdd = !selected->isOdd;
}

void Sidplayer::downButtonClicked()
{
    if ( PlaylistMenu->isItemChecked( allowSortingID ) )
        allowSorting(); // turn column sorting off

    PlaylistItem *selected = (PlaylistItem *) PlaylistView->selectedItem();
    PlaylistItem *below    = (PlaylistItem *) PlaylistView->selectedItem()->itemBelow();
    PlaylistView->selectedItem()->moveItem( below );

    PlaylistView->ensureItemVisible( PlaylistView->selectedItem()->itemAbove() );
    PlaylistView->ensureItemVisible( PlaylistView->selectedItem()->itemBelow() );
    PlaylistView->ensureItemVisible( PlaylistView->selectedItem() );

    checkButtonState();
    playlistChanged = true;

    selected->isOdd = !selected->isOdd;
    below->isOdd    = !below->isOdd;
}

//--------------------------------------------------------------------------------------------

// almost identical to Sidplayer::selectFileItem()
void Sidplayer::selectPlaylistItem( const QString &selectThis )
{
    // find the item in the playlist and select it
    QListViewItemIterator it( PlaylistView );
    for ( ; it.current(); ++it )
    {
        PlaylistItem *item = (PlaylistItem *) it.current();
        if ( item->getFilePath() == selectThis ) // relative path (important)
        {
            PlaylistView->ensureItemVisible( item->itemAbove() );
            PlaylistView->ensureItemVisible( item->itemBelow() );
            PlaylistView->ensureItemVisible( item );
            PlaylistView->setCurrentItem( item ); // Focus
            PlaylistView->setSelected( item, true );
            return;
        }
    }
    // not found
    PlaylistItem *item = (PlaylistItem *) PlaylistView->firstChild();
    PlaylistView->ensureItemVisible( item ); // scroll to the top
    PlaylistView->setCurrentItem( item ); // highlight the first item
    PlaylistView->setSelected( item, false ); // but don't select it
}

void Sidplayer::loadPlaylist()
{
    evenFlag = true;

    QString filename = Global::applicationFileName("sidplayer", "playlist.spl");

    QFile f( filename );
    if ( !f.open( IO_ReadOnly ) )
	return;

    QTextStream ts( &f );
    //ts.setCodec( QTextCodec::codecForName( "UTF-8" ) );

    ts.readLine(); // first line is only a comment

    while( ts.readLine() )
    {
        QFileInfo fi = QFileInfo( ts.readLine() );
        QString name      = ts.readLine();
        QString author    = ts.readLine();
        QString copyright = ts.readLine();
        QString songs     = ts.readLine().stripWhiteSpace(); // for backwards compatibility

        if ( !fi.exists() )
        {
            playlistChanged = true;
            continue;
        }

        PlaylistItem *pli = new PlaylistItem( PlaylistView, PlaylistView->currentItem(), fi,
                                              name,
                                              author,
                                              copyright,
                                              songs );

        PlaylistView->setCurrentItem( pli );
    }

    f.close();

    Config cfg("Sidplayer");
    cfg.setGroup("List");
    selectPlaylistItem( cfg.readEntry( "selectedEntry", "" ) );
    PlaylistView->scrollBy( 0, -8 ); // strange - I have no idea why this is necessary
}

void Sidplayer::savePlaylist()
{
    QString filename = Global::applicationFileName( "sidplayer", "playlist.spl" );

    QFile f( filename );
    if ( !f.open( IO_WriteOnly ) )
	return;

    QTextStream ts( &f );
    //ts.setCodec( QTextCodec::codecForName( "UTF-8" ) );

    ts << "SIDPlayer playlist file" << endl;

    QListViewItemIterator it( PlaylistView );
    for ( ; it.current(); ++it )
    {
        PlaylistItem *pli = (PlaylistItem *) it.current();

        ts << endl
           << pli->getFilePath() << endl
           << pli->text(0) << endl
           << pli->text(1) << endl
           << pli->text(2) << endl
           << pli->text(3) << endl;
    }

    f.close();
}

//--------------------------------------------------------------------------------------------

void Sidplayer::readPlaylistSectionOrder()
{
    Config cfg("Sidplayer");
    cfg.setGroup("List");

    QHeader *header = PlaylistView->header();
    for ( int section = 0; section < header->count(); ++section )
    {
        QString label = header->label( section );
        int index = cfg.readNumEntry( label, section );
        header->moveSection( section, index );
    }
}

void Sidplayer::writePlaylistSectionOrder()
{
    Config cfg("Sidplayer");
    cfg.setGroup("List");

    QHeader *header = PlaylistView->header();
    for ( int section = 0; section < header->count(); ++section )
    {
        QString label = header->label( section );
        int index = header->mapToIndex( section );
        if ( !cfg.hasKey( label ) ||
             cfg.readNumEntry( label ) != index )
            cfg.writeEntry( label, index );
    }
}
