/**************************************************************************************94x78**
**
** This file may be distributed and/or modified under the terms of the GNU General Public
** License version 2 as published by the Free Software Foundation and appearing in the file
** LICENSE.GPL included in the packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE WARRANTY OF DESIGN,
** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
*********************************************************************************************/
#include "volumebarwidget.h"
#include <qpainter.h>

#include <unistd.h>
#include <sys/file.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>

#include <iostream>

VolumeBarWidget::VolumeBarWidget( QWidget *parent = 0, const char *name = 0, WFlags f = 0 )
    : QWidget( parent, name, f )
{
    getVolume();
}

VolumeBarWidget::~VolumeBarWidget()
{}

int VolumeBarWidget::getVolume()
{
    int fd;
    if ( ( fd = open( "/dev/mixer", O_RDWR ) ) >= 0 )
    {
        ioctl( fd, MIXER_READ( 0 ), &volumePercent );
        volumePercent >>= 8;
        ::close( fd );
    }
    else
        volumePercent = 0;

    return volumePercent;
}

void VolumeBarWidget::setVolume( int percent )
{
    // clamp volume percent to be between 0 and 100
    volumePercent = percent = (percent < 0) ? 0 : ((percent > 100) ? 100 : percent);

    int fd;
    if ( ( fd = open( "/dev/mixer", O_RDWR ) ) >= 0 )
    {
        percent |= percent << 8; // set both channels to the same volume
        ioctl( fd, MIXER_WRITE( 0 ), &percent );
        ::close( fd );
    }

    // repaint just the little volume rectangle
    repaint( 1, 1, width() - 2, height() - 2, false );
}

void VolumeBarWidget::paintEvent( QPaintEvent * )
{
    int w = width();
    int h = height();
    int y = h - (h - 2) * volumePercent / 100 - 1;

    QPainter p( this );
    p.fillRect( 1, 1, w - 2, y, colorGroup().base() ); // backgroundColor()
    p.fillRect( 1, y, w - 2, h - y, Qt::red );

    //p.setPen( colorGroup().dark() );
    //p.drawRect( 0, 0, w, h );

    w -= 1;
    h -= 1;
    p.setPen( colorGroup().dark() );
    p.drawLine( 0, 0, w, 0 );
    p.drawLine( 0, 0, 0, h );
    p.setPen( colorGroup().light() );
    p.drawLine( 0, h, w, h );
    p.drawLine( w, 0, w, h );
}

void VolumeBarWidget::mousePressEvent( QMouseEvent *e )
{
    mouseMoveEvent( e );
}

void VolumeBarWidget::mouseMoveEvent( QMouseEvent *e )
{
    setVolume( 102 - e->y() * 100 / ( height() - 2 ) );
}
